from plone import api
from zope.annotation import IAnnotations
from zope.interface import Interface
from zope.interface.declarations import implementer

from Products.EasyNewsletter.config import AGG_SOURCES_INFOS
from Products.Five.browser import BrowserView


class IENLHelperView(Interface):
    """ """

    def get_scale_util(self, brain=None):
        """
        return the scale_util for a brain's object,
        if the object does not have an image field, return None
        """

    def get_object_url(self, brain):
        """return object url or None"""

    def brain_has_lead_image(self, brain=None):
        """check if brain has lead image"""

    def type_filter(self, items, types=None):
        """filter given list by portal_type"""

    def get_issue(self, context):
        """return the issue obj"""

    def get_results_from_aggregation_sources(self, context):
        """return a list of result sets based on aggregation sources"""


@implementer(IENLHelperView)
class ENLHelperView(BrowserView):
    """View with some helper methods"""

    def get_scale_util(self, brain=None):
        if not brain:
            return
        context = brain.getObject()
        has_image = hasattr(context.aq_explicit, "image")
        if not has_image:
            return
        scale_util = api.content.get_view("images", context)
        return scale_util

    def get_object_url(self, brain):
        """return object url or None"""
        if not brain:
            return
        return brain.getURL()

    def brain_has_lead_image(self, brain=None):
        has_image = False
        if not brain:
            return
        item_object = brain.getObject()
        # Plone 5:
        has_image = hasattr(item_object.aq_explicit, "image")
        return has_image

    def type_filter(self, items, types=None):
        """filter given list by portal_type"""
        if not types:
            return items
        allowed_items = []
        for item in items:
            if item.portal_type not in types:
                continue
            allowed_items.append(item)
        return allowed_items

    def get_issue(self, context):
        return api.content.get(UID=context.issue_uid)

    def get_results_from_aggregation_sources(self, context):
        """Returns a list of sources and it's results"""
        return self._get_source_infos_from_annotation(context)

    def _get_source_infos_from_annotation(self, context):
        """ """
        annotations = IAnnotations(context)
        if AGG_SOURCES_INFOS not in annotations:
            return []
        return annotations[AGG_SOURCES_INFOS]



from hatchling.metadata.plugin.interface import MetadataHookInterface


class CustomMetadataHook(MetadataHookInterface):
    PLUGIN_NAME = "custom"

    def update(self, metadata: dict) -> None:
        with open("README.md", encoding="utf-8") as f:
            readme = f.read()
        with open("CHANGELOG.md", encoding="utf-8") as f:
            changelog = f.read()
        metadata["readme"] = {
            "content-type": "text/markdown",
            "text": readme + "\n\n" + changelog,
        }



from logging import getLogger

import emails
import OFS
from AccessControl.SecurityManagement import newSecurityManager
from Acquisition import aq_inner
from email_validator import EmailNotValidError, validate_email
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zExceptions import BadRequest
from zope.component import getMultiAdapter, queryUtility
from zope.interface import alsoProvides

from Products.CMFCore.utils import getToolByName
from Products.EasyNewsletter import EasyNewsletterMessageFactory as _  # noqa
from Products.EasyNewsletter.config import MESSAGE_CODE
from Products.EasyNewsletter.content.newsletter import INewsletter
from Products.EasyNewsletter.content.newsletter_subscriber import INewsletterSubscriber
from Products.EasyNewsletter.interfaces import IENLRegistrationTool
from Products.EasyNewsletter.utils.base import execute_under_special_role
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage

try:
    from plone import protect
except ImportError:
    # BBB for old plone.protect, default until at least Plone 4.3.7.
    protect = None


logger = getLogger("ENL Registration")


class SubscriberView(BrowserView):
    """ """

    def _msg_redirect(self, newsletter):
        """To also display messages for anon users"""
        if api.user.is_anonymous():
            return self.request.response.redirect(self.context.absolute_url())
        else:
            self.request.response.redirect(newsletter.absolute_url())

    def portal_state(self):
        context = aq_inner(self.context)
        return getMultiAdapter((context, self.request), name="plone_portal_state")

    @property
    def portal(self):
        pstate = self.portal_state()
        return pstate.portal()

    @property
    def portal_url(self):
        pstate = self.portal_state()
        return pstate.portal_url()

    def register_subscriber(self):
        """ """
        messages = IStatusMessage(self.request)
        path_to_easynewsletter = self.request.get("newsletter")
        # remove leading slash from paths like: /mynewsletter
        path_to_easynewsletter = path_to_easynewsletter.strip("/")
        newsletter_container = self.portal.unrestrictedTraverse(path_to_easynewsletter)

        subscriber = self.request.get("subscriber")
        try:
            valid_subscriber = validate_email(subscriber)
        except EmailNotValidError as e:
            messages.addStatusMessage(_(f"Please enter a valid email address.\n{e}"), "error")
            return self._msg_redirect(newsletter_container)

        lastname = self.request.get("name", "")
        firstname = self.request.get("firstname", "")
        name_prefix = self.request.get("name_prefix", "")
        portal_state = getMultiAdapter(
            (self.context.aq_inner, self.request), name="plone_portal_state"
        )
        current_language = portal_state.language()
        nl_language = self.request.get("nl_language", current_language)
        salutation = self.request.get("salutation", "")
        organization = self.request.get("organization", "")

        norm = queryUtility(IIDNormalizer)
        normalized_subscriber = norm.normalize(valid_subscriber.email)
        if normalized_subscriber in newsletter_container.objectIds():
            messages.addStatusMessage(_("Your email address is already registered."), "error")
            return self._msg_redirect(newsletter_container)
        subscriber_data = {}
        subscriber_data["subscriber"] = valid_subscriber.email
        subscriber_data["lastname"] = lastname
        subscriber_data["firstname"] = firstname
        subscriber_data["name_prefix"] = name_prefix
        subscriber_data["nl_language"] = nl_language
        subscriber_data["salutation"] = salutation
        subscriber_data["organization"] = organization
        subscriber_data["path_to_easynewsletter"] = path_to_easynewsletter

        # use password reset tool to create a hash
        pwr_data = self._requestReset(subscriber_data["subscriber"])
        hashkey = pwr_data["randomstring"]
        enl_registration_tool = queryUtility(IENLRegistrationTool, "enl_registration_tool")
        if hashkey not in enl_registration_tool.objectIds():
            enl_registration_tool[hashkey] = RegistrationData(hashkey, **subscriber_data)
            msg_subject = newsletter_container.subscriber_confirmation_mail_subject.replace(
                "${portal_url}", self.portal_url.strip("http://")
            )
            confirmation_url = self.portal_url + "/confirm-subscriber?hkey=" + str(hashkey)
            confirmation_url = protect.utils.addTokenToUrl(confirmation_url)
            msg_text = newsletter_container.subscriber_confirmation_mail_text.replace(
                "${newsletter_title}", newsletter_container.title
            )
            msg_text = msg_text.replace("${subscriber_email}", subscriber_data["subscriber"])
            msg_text = msg_text.replace("${confirmation_url}", confirmation_url)
            settings = get_portal_mail_settings()
            msg_sender = settings.email_from_address
            msg_receiver = subscriber
            msg = emails.Message(
                text=msg_text,
                subject=msg_subject,
                mail_from=msg_sender,
                mail_to=msg_receiver,
            )
            self.portal.MailHost.send(msg.as_string())

            messages.addStatusMessage(
                _(
                    "Your email has been registered. \
                A confirmation email was sent to your address. Please check \
                your inbox and click on the link in the email in order to \
                confirm your subscription."
                ),
                "info",
            )
            return self._msg_redirect(newsletter_container)

    def confirm_subscriber(self):
        hashkey = self.request.get("hkey")
        enl_registration_tool = queryUtility(IENLRegistrationTool, "enl_registration_tool")
        regdataobj = enl_registration_tool.get(hashkey)
        messages = IStatusMessage(self.request)
        if regdataobj:
            portal = api.portal.get()
            easynewsletter = portal.unrestrictedTraverse(regdataobj.path_to_easynewsletter)
            email = regdataobj.subscriber
            plone_utils = api.portal.get_tool(name="plone_utils")
            subscriber_id = plone_utils.normalizeString(email)
            try:
                execute_under_special_role(
                    portal,
                    "Contributor",
                    api.content.create,
                    type="Newsletter Subscriber",
                    id=subscriber_id,
                    language=self.context.language,
                    email=email,
                    firstname=regdataobj.firstname,
                    lastname=regdataobj.lastname,
                    name_prefix=regdataobj.name_prefix,
                    # nl_language=regdataobj.nl_language,
                    organization=regdataobj.organization,
                    salutation=regdataobj.salutation,
                    container=easynewsletter,
                )
            except BadRequest:
                error_code = "email_exists"
                messages.addStatusMessage(MESSAGE_CODE[error_code], "error")
            else:
                # now delete the regobj
                del enl_registration_tool[hashkey]
                messages.addStatusMessage(
                    _("Your subscription was successfully confirmed."), "info"
                )
            return self._msg_redirect(easynewsletter)
        else:
            messages.addStatusMessage(_("Please enter a valid email address."), "error")
        return self.request.response.redirect(self.context.absolute_url())

    def _requestReset(self, userid):  # noqa
        """Ask the system to start the password reset procedure for
        user 'userid'.

        Returns a dictionary with the random string that must be
        used to reset the password in 'randomstring', the expiration date
        as a DateTime in 'expires', and the userid (for convenience) in
        'userid'.
        # taken from Products.PasswordResetTool but without getValidUser check!
        """
        pwrtool = getToolByName(self.context, "portal_password_reset")
        randomstring = pwrtool.uniqueString(userid)
        expiry = pwrtool.expirationDate()
        pwrtool._requests[randomstring] = (userid, expiry)
        # clear out untouched records > 10 days old this is a cheap sort of
        # automatic clearing
        pwrtool.clearExpired(10)
        pwrtool._p_changed = 1
        retval = {}
        retval["randomstring"] = randomstring
        retval["expires"] = expiry
        retval["userid"] = userid
        return retval


class RegistrationData(OFS.SimpleItem.Item):
    """holds data from ENL registration form"""

    def __init__(self, id, **kw):
        self.id = id
        for key, value in kw.items():
            setattr(self, key, value)
        # super(RegistrationData, self).__init__()

    @property
    def title(self):
        return "%s - %s" % (" ".join([self.firstname, self.lastname]), self.subscriber)


class UnsubscribeView(BrowserView):
    def __call__(self):
        self.newsletter_url = self.context.absolute_url()
        subscriber = self.request.get("subscriber")

        if subscriber:
            self.send_unsubscribe_email(subscriber)
            return self.request.response.redirect(self.newsletter_url)
        else:
            self.form_action = self.newsletter_url + "/unsubscribe"
            return self.index()

    def send_unsubscribe_email(self, subscriber):
        newsletter = self.context
        catalog = getToolByName(self.context, "portal_catalog")
        query = {}
        query["portal_type"] = "Newsletter Subscriber"
        query["email"] = subscriber
        results = catalog.unrestrictedSearchResults(query)
        messages = IStatusMessage(self.request)
        if results:
            subscriber_brain = results[0]
            unsubscribe_url = (
                self.newsletter_url + "/unsubscribe?subscriber=" + subscriber_brain.UID
            )
            msg_text = """%s: %s""" % (newsletter.unsubscribe_string, unsubscribe_url)
            settings = get_portal_mail_settings()
            api.portal.send_email(
                recipient=subscriber,
                sender=settings.email_from_address,
                subject=_("confirm newsletter unsubscription"),
                body=msg_text,
            )
            messages.addStatusMessage(
                _("We send you an email, please confirm this unsubscription."), "info"
            )
        else:
            # todo: write an extra error msg if a plone user wants to
            # unsubscribe himself
            messages.addStatusMessage(
                _("Your email address could not be found in subscribers."), "error"
            )

    def unsubscribe(self):
        """ """
        if protect is not None:
            alsoProvides(self.request, protect.interfaces.IDisableCSRFProtection)
        putils = getToolByName(self.context, "plone_utils")
        uid = self.request.get("subscriber")

        newsletter = self.context
        if not INewsletter.providedBy(newsletter):
            putils.addPortalMessage(_("Please use the correct unsubscribe url!"), "error")
            return self.request.response.redirect(
                api.portal.get_navigation_root(self).absolute_url()
            )

        # We do the deletion as the owner of the newsletter object
        # so that this is possible without login.
        owner = newsletter.getWrappedOwner()
        newSecurityManager(newsletter, owner)
        subscriber = api.content.get(UID=uid)
        if subscriber is None or not INewsletterSubscriber.providedBy(subscriber):
            putils.addPortalMessage(_("An error occured"), "error")
        else:
            del newsletter[subscriber.id]
            putils.addPortalMessage(_("You have been unsubscribed."))

        return self.request.response.redirect(api.portal.get_navigation_root(self).absolute_url())



import transaction
from plone import api
from plone.protect import PostOnly
from plone.protect.interfaces import IDisableCSRFProtection
from zope.interface import alsoProvides

from Products.CMFPlone.resources import add_resource_on_request
from Products.EasyNewsletter import EasyNewsletterMessageFactory as _  # noqa
from Products.EasyNewsletter.interfaces import IIssueDataFetcher
from Products.Five.browser import BrowserView


class IssueView(BrowserView):
    """Single Issue View"""

    def __call__(self):
        add_resource_on_request(self.request, "iframeResizer")
        return super().__call__()

    @property
    def here_url(self):
        return self.context.absolute_url()

    # TODO: >> new AggregateContent view
    def refresh_issue(self, REQUEST=None):  # noqa
        """Refresh the aggregate body when using collections."""
        alsoProvides(self.request, IDisableCSRFProtection)
        self.context.loadContent()
        self.request.response.redirect(self.context.absolute_url())

    # TODO: >> SendIssue view
    def _send_issue_prepare(self):
        self.request["enlwf_guard"] = True
        api.content.transition(obj=self.context, transition="send")
        # commit the transaction so that identical incoming requests, for
        # whatever reason, will not trigger another send
        transaction.commit()
        self.request["enlwf_guard"] = False

    # TODO: >> SendIssue view
    def send_issue(self):
        """
        sets workflow state to sending and then redirects to step2 with UID as
        parameter as simple safety belt.
        """
        PostOnly(self.request)
        if "test" in self.request.form:  # test must not modify the state
            self.context.send()
            api.portal.show_message(
                message=_("The issue test sending has been initiated."),
                request=self.request,
            )
            return self.request.response.redirect(self.context.absolute_url())

        if self.context.issue_queue is not None:
            self._send_issue_prepare()
            self.context.queue_issue_for_sendout()
            api.portal.show_message(
                message=_("The issue sending has been initiated in the background."),
                request=self.request,
            )
            return self.request.response.redirect(self.context.absolute_url())

        # No queuing but direct send
        self.send_issue_immediately()
        api.portal.show_message(
            message=_("The issue has been generated and sent to the mail server."),
            request=self.request,
        )
        return self.request.response.redirect(self.context.absolute_url())

    # TODO: >> SendIssueImmediately view
    def send_issue_immediately(self):
        """convinience view for cron and similar

        never call this from UI - needs a way to protect
        currently manager only
        """
        self._send_issue_prepare()
        self.context.send()

    def get_public_body(self):
        """Return the rendered HTML version without placeholders."""
        issuedatafetcher = IIssueDataFetcher(self.context)
        preview_html = issuedatafetcher.preview_html()
        return preview_html

    def get_preview(self):
        """Return the rendered HTML version with all placeholder,
        for admin preview.
        """
        test_receiver = {
            "email": "john@example.com",
            "fullname": "John Doe",
            "firstname": "John",
            "lastname": "Doe",
            "salutation": "Dear Mr.",
            "nl_language": "de",
            "uid": "xyz",
        }
        issuedatafetcher = IIssueDataFetcher(self.context)
        preview_html = issuedatafetcher.preview_html(disable_filter=True, receiver=test_receiver)
        return preview_html

    # BBB: can be removed in 5.x
    def copy_as_draft(self):
        newsletter = self.context.aq_parent
        master_id = self.context.getId()

        if master_id.startswith("master_"):
            draft_id = master_id.replace("master_", "")
        else:
            draft_id = master_id

        draft_obj = api.content.copy(
            source=self.context, target=newsletter, safe_id=True, id=draft_id
        )

        return self.request.response.redirect(draft_obj.absolute_url() + "/edit")

    # BBB: can be removed in 5.x
    def copy_as_master(self):
        request = self.context.REQUEST
        newsletter = self.context.aq_parent
        master_id = "master_" + self.context.getId()

        master_obj = api.content.copy(
            source=self.context, target=newsletter, safe_id=True, id=master_id
        )

        request["enlwf_guard"] = True
        api.content.transition(obj=master_obj, transition="make_master")
        request["enlwf_guard"] = False

        return self.request.response.redirect(master_obj.absolute_url() + "/edit")



from plone.app.standardtiles.common import ProxyViewletTile


class NewsletterActionsTile(ProxyViewletTile):
    """Newsletter actions tile."""

    manager = "plone.abovecontentbody"
    viewlet = "newsletter-actions"



import unittest

import six
from plone import api
from plone.app.testing import TEST_USER_ID, setRoles
from zope.component import getMultiAdapter
from zope.interface.interfaces import ComponentLookupError

from Products.EasyNewsletter.testing import (
    PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING,
    PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING,
)


class ViewsIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.newsletter = api.content.create(
            container=self.portal, type="Newsletter", id="newsletter"
        )
        self.newsletter.subscriber = api.content.create(
            container=self.newsletter,
            type="Newsletter Subscriber",
            id="newsletter_subscriber",
            email="jane@example.com",
            lastname="Doe",
            firstname="Jane",
            organization="Doe Cörp",
        )

    def test_subscribers_download_is_registered(self):
        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="subscribers-download")
        self.assertTrue(view.__name__ == "subscribers-download")

    def test_subscribers_download_not_matching_interface(self):
        with self.assertRaises(ComponentLookupError):
            getMultiAdapter((self.portal, self.portal.REQUEST), name="subscribers-download")

    def test_subscribers_download_call(self):
        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="subscribers-download")
        result = view()
        self.assertEqual(
            b"salutation,name_prefix,firstname,lastname,email,organization\n,,Jane,Doe,jane@example.com,Doe C\xc3\xb6rp\n",
            result.replace(b"\r", b""),
        )
        self.assertTrue(isinstance(result, six.binary_type), "result must be binary_type!")


class ViewsFunctionalTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



import OFS


class ENLRegistrationTool(OFS.Folder.Folder):
    """ """

    id = "enl_registration_tool"
    title = "ENL registration utility"
    meta_type = "enl_registration_tool"
    portal_type = meta_type



import logging

from plone import api, schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope.component import adapter
from zope.interface import Interface, implementer, provider
from zope.schema.interfaces import IContextAwareDefaultFactory

from Products.EasyNewsletter import _

from ..content.newsletter import INewsletter

log = logging.getLogger("Products.EasyNewsletter")


@provider(IContextAwareDefaultFactory)
def get_default_plone_receiver_members(parent):
    if not INewsletter.providedBy(parent) or not parent.__parent__:
        return set()
    return parent.plone_receiver_members


@provider(IContextAwareDefaultFactory)
def get_default_plone_receiver_groups(parent):
    if not INewsletter.providedBy(parent) or not parent.__parent__:
        return set()
    return parent.plone_receiver_groups


class IPloneUserGroupRecipientsMarker(Interface):
    pass


@provider(IFormFieldProvider)
class IPloneUserGroupRecipients(model.Schema):
    """ """

    model.fieldset(
        "recipients",
        label=_("Recipients"),
        fields=["plone_receiver_members", "plone_receiver_groups"],
    )

    plone_receiver_members = schema.Set(
        title=_(
            "EasyNewsletter_label_ploneReceiverMembers",
            default="Plone Members to receive the newsletter",
        ),
        description=_(
            "EasyNewsletter_help_ploneReceiverMembers",
            default="Choose Plone Members which should receive \
                    the newsletter. Changing this setting does not affect \
                    already existing issues.",
        ),
        value_type=schema.Choice(vocabulary="Products.EasyNewsletter.PloneUsers"),
        required=False,
        defaultFactory=get_default_plone_receiver_members,
    )

    plone_receiver_groups = schema.Set(
        title=_(
            "EasyNewsletter_label_ploneReceiverGroups",
            default="Plone Groups to receive the newsletter",
        ),
        description=_(
            "EasyNewsletter_help_ploneReceiverGroups",
            default="Choose Plone Groups which should receive \
                    the newsletter. Changing this setting does not affect \
                    already existing issues.",
        ),
        value_type=schema.Choice(vocabulary="Products.EasyNewsletter.PloneGroups"),
        required=False,
        defaultFactory=get_default_plone_receiver_groups,
    )


@implementer(IPloneUserGroupRecipients)
@adapter(IPloneUserGroupRecipientsMarker)
class PloneUserGroupRecipients:
    def __init__(self, context):
        self.context = context

    @property
    def plone_receiver_members(self):
        return getattr(self.context, "plone_receiver_members", set())

    @plone_receiver_members.setter
    def plone_receiver_members(self, value):
        self.context.plone_receiver_members = value

    @property
    def plone_receiver_groups(self):
        return getattr(self.context, "plone_receiver_groups", set())

    @plone_receiver_groups.setter
    def plone_receiver_groups(self, value):
        self.context.plone_receiver_groups = value

    def _get_salutation_mappings(self):
        """
        returns mapping of salutations. Each salutation itself is a dict
        with key as language. (prepared for multilingual newsletter)
        """
        enl = self.context.get_newsletter()
        result = {}
        lang = self.context.language or "en"

        for line in enl.salutations:
            if "|" not in line:
                continue
            key, value = line.split("|")
            result[key.strip()] = {lang: value.strip()}
        return result

    def get_plone_subscribers(self):
        """Search for all selected Members and Groups
        and return a filtered list of subscribers as dicts.
        """
        plone_subscribers = []
        receiver_list = set()
        receiver_member_list = self.plone_receiver_members
        receiver_group_list = self.plone_receiver_groups

        for member_id in receiver_member_list:
            try:
                member = api.user.get(userid=member_id)
            except (AttributeError, api.exc.PloneApiError):
                continue
            else:
                receiver_list.add(member)

        for group in receiver_group_list:
            try:
                members = api.user.get_users(groupname=group)
            except (AttributeError, api.exc.PloneApiError):
                continue
            else:
                receiver_list.update(members)

        # get salutation mappings
        salutation_mappings = self._get_salutation_mappings()

        # get all selected member properties
        for member in receiver_list:
            email = member.getProperty("email")
            if not email:
                continue
            salutation = salutation_mappings[member.getProperty("nl_gender", "default")]
            language = member.getProperty("language") or self.context.language
            plone_subscribers.append({
                "userid": member.getUserId(),
                "fullname": member.getProperty("fullname"),
                "email": email,
                "salutation": salutation.get(
                    language,
                    salutation.get(self.context.language or "en", "unset"),
                ),
                "nl_language": language,
            })
        return plone_subscribers



import unittest

import transaction
from Acquisition import aq_base
from plone import api
from plone.app.testing import TEST_USER_ID, setRoles
from z3c.relationfield.relation import RelationValue
from zExceptions import BadRequest
from zope.component import getMultiAdapter, getSiteManager, getUtility
from zope.intid.interfaces import IIntIds

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.tests.utils import MockMailHost
from Products.EasyNewsletter.content.newsletter_issue import INewsletterIssue
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.MailHost.interfaces import IMailHost


class DailyIssueBaseTestCase(unittest.TestCase):
    """Test case sending a daily Newsletter issue"""

    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.catalog = getToolByName(self.portal, "portal_catalog")
        setRoles(self.portal, TEST_USER_ID, ["Manager", "Contributor", "Owner", "Editor"])

        # creating test objects: folder, news, newsletter and subscriber
        self.portal.invokeFactory("Folder", "testfolder")
        self.folder = self.portal["testfolder"]
        self.folder.invokeFactory("News Item", "news01")

        self.folder.invokeFactory("Newsletter", "daily-news")
        self.newsletter = self.folder["daily-news"]
        self.newsletter.title = "Daily News"
        # XXX check if we could avoid this by using defaults from site settings
        self.newsletter.sender_email = "newsletter@acme.com"
        self.newsletter.sender_name = "ACME newsletter"
        self.newsletter.test_email = "test@acme.com"

        news_collection = api.content.create(
            type="Collection",
            id="news-collection",
            title="News Collection",
            container=self.folder,
        )
        query = [
            {
                "i": "portal_type",
                "o": "plone.app.querystring.operation.selection.is",
                "v": ["News Item"],
            }
        ]
        news_collection.setQuery(query)
        news_collection.aggregation_template = "aggregation_generic_listing"
        transaction.commit()
        intids = getUtility(IIntIds)
        to_id = intids.getId(news_collection)
        self.newsletter.content_aggregation_sources = [RelationValue(to_id)]

        api.content.create(
            type="Newsletter Subscriber",
            id="subscriber01",
            container=self.newsletter,
            email="jane@example.com",
        )
        self.view = getMultiAdapter((self.newsletter, self.layer["request"]), name="daily-issue")

        self.mail_settings = get_portal_mail_settings()
        self.mail_settings.email_from_address = "noreply@plone.org"
        # setting a Mock mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)

    def tearDown(self):
        self.portal.MailHost = self.portal._original_MailHost
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(aq_base(self.portal._original_MailHost), provided=IMailHost)


class DailyIssueContent(DailyIssueBaseTestCase):
    def test_create_new_issue(self):
        issues = self.catalog(
            object_provides=INewsletterIssue.__identifier__,
            path="/".join(self.newsletter.getPhysicalPath()),
        )
        self.assertEqual(len(issues), 0)
        self.assertFalse(self.view.already_sent())
        self.view.create_issue()

        issues = self.catalog(
            object_provides=INewsletterIssue.__identifier__,
            path="/".join(self.newsletter.getPhysicalPath()),
        )

        self.assertTrue(self.view.already_sent())
        self.assertEqual(len(issues), 1)
        self.assertEqual(self.view.issue.title, "Daily News")

    def test_empty_issue(self):
        self.assertTrue(self.view.has_content())
        self.folder.manage_delObjects(["news01"])
        self.assertFalse(self.view.has_content())

    def test_send_issue(self):
        try:
            self.view.create_issue()
        except Exception:
            self.fail("Couldn't create issue!")

        self.view.send()
        self.assertEqual(len(self.portal.MailHost.messages), 1)


class DailyIssueMethodGET(DailyIssueBaseTestCase):
    def setUp(self):
        self.layer["request"]["REQUEST_METHOD"] = "GET"
        DailyIssueBaseTestCase.setUp(self)

    def test_get_with_an_empty_issue(self):
        self.folder.manage_delObjects(["news01"])
        self.view()
        self.assertEqual(self.view.request.response.getStatus(), 204)

    def test_get_with_a_non_empty_issue(self):
        self.view()
        self.assertEqual(self.view.request.response.getStatus(), 100)

    def test_get_an_alredy_sent_issue(self):
        self.view.create_issue()
        self.view()
        self.assertEqual(self.view.request.response.getStatus(), 200)


class DailyIssueMethodPOST(DailyIssueBaseTestCase):
    def setUp(self):
        self.layer["request"]["REQUEST_METHOD"] = "POST"
        DailyIssueBaseTestCase.setUp(self)

    def test_do_not_create_or_send_an_empty_issue(self):
        self.folder.manage_delObjects(["news01"])
        self.view()
        issues = self.catalog(
            object_provides=INewsletterIssue.__identifier__,
            path="/".join(self.newsletter.getPhysicalPath()),
        )
        self.assertFalse(issues)
        self.assertEqual(self.view.request.response.getStatus(), 204)
        self.assertEqual(len(self.portal.MailHost.messages), 0)

    def test_send_issue_and_check_http_status(self):
        print(api.user.getPermissions())
        import pdb

        pdb.set_trace()  # NOQA: E702
        self.view()
        self.assertEqual(self.view.request.response.getStatus(), 200)
        self.assertEqual(len(self.portal.MailHost.messages), 1)

    def test_do_not_send_same_issue_twice(self):
        self.view()  # 200 OK
        self.assertEqual(self.view.request.response.getStatus(), 200)
        self.assertRaises(BadRequest, self.view.create_issue)
        self.view()  # 409 Already Sent
        self.assertEqual(self.view.request.response.getStatus(), 409)


class TriggerDailyIssueMethodGET(DailyIssueBaseTestCase):
    def setUp(self):
        self.layer["request"]["REQUEST_METHOD"] = "GET"
        DailyIssueBaseTestCase.setUp(self)
        self.view = getMultiAdapter(
            (self.newsletter, self.layer["request"]), name="trigger-daily-issue"
        )

    def test_send_issue_and_check_http_status(self):
        self.view()
        self.assertEqual(self.view.request.response.getStatus(), 200)
        self.assertEqual(len(self.portal.MailHost.messages), 1)


class DailyIssueMethodOtherThanGETorPOST(DailyIssueBaseTestCase):
    def setUp(self):
        self.layer["request"]["REQUEST_METHOD"] = "FOOBAR"
        DailyIssueBaseTestCase.setUp(self)

    def test_trying_another_method_on_view(self):
        self.view()
        self.assertEqual(self.view.request.response.getStatus(), 405)
        self.assertEqual(self.view.request.response.getHeader("Allow"), "GET, POST")


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



from Products.CMFCore.utils import getToolByName
from Products.EasyNewsletter.config import ENL_ISSUE_TYPES
from Products.Five.browser import BrowserView


class NewsletterView(BrowserView):
    """ """

    def getRenderedIssues(self):
        """Return the rendered body of all newsletter issues"""

        result = list()
        catalog = getToolByName(self.context, "portal_catalog")
        for brain in catalog(
            portal_type=ENL_ISSUE_TYPES, path="/".join(self.context.getPhysicalPath())
        ):
            issue = brain.getObject()
            content = issue.restrictedTraverse("@@get-public-body")()
            result.append(dict(content=content, title=issue.Title()))
        return result



import unittest

from App.Common import package_home
from plone import api
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, setRoles
from plone.registry.interfaces import IRegistry
from zope.component import getGlobalSiteManager, getSiteManager, getUtility, provideHandler

from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.tests.utils import MockMailHost
from Products.EasyNewsletter.interfaces import IBeforePersonalizationEvent, IIssueDataFetcher
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.MailHost.interfaces import IMailHost

GLOBALS = globals()
TESTS_HOME = package_home(GLOBALS)


class IssuedatafetcherIntegrationTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.portal_url()
        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.newsletter = api.content.create(
            container=self.portal,
            type="Newsletter",
            id="enl1",
            title="ENL 1",
            sneder_email="newsletter@acme.com",
            sender_name="ACME newsletter",
            test_email="test@acme.com",
        )
        self.mail_settings = get_portal_mail_settings()
        # Set up a mock mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        self.portal.MailHost.smtp_host = "localhost"
        registry = getUtility(IRegistry)
        self.mail_settings = registry.forInterface(IMailSchema, prefix="plone")
        self.mail_settings.email_from_address = "portal@plone.test"
        self.mail_settings.smtp_host = "localhost"
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        # We need to fake a valid mail setup
        self.mailhost = self.portal.MailHost
        api.content.create(
            container=self.newsletter,
            type="Newsletter Issue",
            id="issue",
            title="This is a very long newsletter issue title with special "
            "characters such as äüö. Will this really work?",
        )

    def test_before_personalization_filter(self):
        def personalize(event):
            edc = event.data["context"]
            event.data["html"] = event.data["html"].replace("PHP", "Python")
            edc["SUBSCRIBER_SALUTATION"] = "Dear Ms. Jane Doe"

        provideHandler(personalize, [IBeforePersonalizationEvent])

        try:
            receiver = {
                "email": "john@example.com",
                "fullname": "John Doe",
                "firstname": "John",
                "lastname": "Doe",
                "salutation": "Dear Mr.",
                # "nl_language": "de",
            }
            html = """
            <h1>PHP ist toll!</h1>
            {{SUBSCRIBER_SALUTATION}}
            """
            issue_data_fetcher = IIssueDataFetcher(self.newsletter.issue)
            issue_data = issue_data_fetcher.personalize(receiver, html)
            self.assertIn("Dear Ms. Jane Doe", issue_data)
        finally:
            getGlobalSiteManager().unregisterHandler(personalize, [IBeforePersonalizationEvent])

    def test_fetching_issue_data(self):

        receiver = {
            "email": "john@example.com",
            "fullname": "John Doe",
            "firstname": "John",
            "lastname": "Doe",
            "salutation": "Dear Mr.",
        }
        html = """
        <h1></h1>
        {{SUBSCRIBER_SALUTATION}}
        """
        issue_data_fetcher = IIssueDataFetcher(self.newsletter.issue)
        issue_data = issue_data_fetcher.personalize(receiver, html)
        self.assertIn("Dear Mr. John Doe", issue_data)



from plone import api
from plone.app.layout.viewlets import ViewletBase


class NewsletterActions(ViewletBase):
    def update(self):
        """ """
        self.id = self.context.id
        self.enl_url = self.context.absolute_url()
        self.view_name = self.view.__name__

    def render(self):
        if not api.user.is_anonymous():
            return super().render()
        else:
            return ""



from AccessControl import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager, setSecurityManager
from AccessControl.User import Super as BaseUnrestrictedUser

from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter.utils.mail import get_email_charset


def safe_portal_encoding(string):
    charset = get_email_charset()
    return safe_unicode(string).encode(charset)


def execute_under_special_role(portal, role, function, *args, **kwargs):
    """ Execute code under special role priviledges.
    Example how to call::
        execute_under_special_role(portal, "Manager",
            doSomeNormallyNotAllowedStuff,
            source_folder, target_folder)

    @param portal: Reference to ISiteRoot obj whose access ctls we are using
    @param function: Method to be called with special priviledges
    @param role: User role we are using for the security context when calling \
                 the priviledged code. For example, use "Manager".
    @param args: Passed to the function
    @param kwargs: Passed to the function
    """

    sm = getSecurityManager()
    try:
        try:
            # Clone the current access control user and assign a new role
            # for him/her. Note that the username (getId()) is left in
            # exception tracebacks in error_log
            # so it is important thing to store
            tmp_user = UnrestrictedUser(sm.getUser().getId(), "", [role], "")

            # Act as user of the portal
            tmp_user = tmp_user.__of__(portal.acl_users)
            newSecurityManager(None, tmp_user)

            # Call the function
            return function(*args, **kwargs)

        except Exception:
            # If special exception handlers are needed, run them here
            raise
    finally:
        # Restore the old security manager
        setSecurityManager(sm)


class UnrestrictedUser(BaseUnrestrictedUser):
    """Unrestricted user that still has an id."""

    def getId(self):
        """Return the ID of the user."""
        return self.getUserName()



from plone import api
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from Products.EasyNewsletter.content.newsletter import INewsletter


@implementer(IVocabularyFactory)
class NewslettersVocabulary:
    """Provide available newsletters."""

    def __call__(self, context):
        terms = []
        newsletters = api.content.find(object_provides=INewsletter)
        for newsletter in newsletters:
            terms.append(
                SimpleTerm(
                    value=newsletter.getPath(),
                    token=str(newsletter.getPath()),
                    title=newsletter.getPath(),
                )
            )

        return SimpleVocabulary(terms)


NewslettersVocabularyFactory = NewslettersVocabulary()



import codecs
import csv
import tempfile
from logging import getLogger

import cStringIO
import six
from Acquisition import aq_inner
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.protect.utils import addTokenToUrl
from zope.component import getUtility
from zope.interface import Interface, implementer
from zope.interface.interfaces import ComponentLookupError

from Products.CMFCore.utils import getToolByName
from Products.EasyNewsletter import EasyNewsletterMessageFactory as _
from Products.EasyNewsletter.config import SALUTATION
from Products.EasyNewsletter.interfaces import ISubscriberSource
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage

logger = getLogger("Subscribers")


class UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """

    def __init__(self, f, encoding):
        self.reader = codecs.getreader(encoding)(f)

    def __iter__(self):
        return self

    def next(self):
        return self.reader.next().encode("utf-8")


class UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def next(self):
        row = self.reader.next()
        return [six.text_type(s, "utf-8") for s in row]

    def __iter__(self):
        return self


class UnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        # Redirect output to a queue
        self.queue = cStringIO.StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        self.writer.writerow([s.encode("utf-8") for s in row])
        # Fetch UTF-8 output from the queue ...
        data = self.queue.getvalue()
        data = data.decode("utf-8")
        # ... and reencode it into the target encoding
        data = self.encoder.encode(data)
        # write to the target stream
        self.stream.write(data)
        # empty queue
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)


def normalize_id(astring):
    return getUtility(IIDNormalizer).normalize(astring)


CSV_HEADER = [
    "salutation",
    "name_prefix",
    "firstname",
    "lastname",
    "nl_language",
    "email",
    "organization",
]


class IEnl_Subscribers_View(Interface):
    """
    Enl_Subscribers_View interface
    """


@implementer(IEnl_Subscribers_View)
class Enl_Subscribers_View(BrowserView):
    """
    Enl_Subscribers_View browser view
    """

    # TODO: we should move these indexes from FieldIndex to ZCTextIndex
    # see setuphandlers.py for indexes creation
    searchable_params = ("SearchableText",)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        if self.can_delete():
            self.delete()
        return super().__call__()

    @property
    def portal_catalog(self):
        return getToolByName(self.context, "portal_catalog")

    @property
    def portal(self):
        return getToolByName(self.context, "portal_url").getPortalObject()

    @property
    def query(self):
        query = dict(
            portal_type="ENLSubscriber",
            path="/".join(self.context.getPhysicalPath()),
            sort_on="email",
        )
        form = self.request.form
        for k in self.searchable_params:
            if form.get(k):
                if k == "SearchableText":
                    searchterm = form.get(k)
                    if not searchterm.endswith("*"):
                        searchterm += "*"
                    query[k] = searchterm
                else:
                    query[k] = form.get(k)
        return query

    def subscribers(self):
        subscribers = list()

        # Plone subscribers
        for brain in self.portal_catalog(self.query):
            if brain.salutation:
                salutation = SALUTATION.getValue(brain.salutation, "")
            else:
                salutation = ""
            subscribers.append(
                dict(
                    id=brain.getId,
                    source="plone",
                    deletable=True,
                    email=brain.email,
                    getURL=brain.getURL(),
                    salutation=salutation,
                    name_prefix=brain.name_prefix,
                    firstname=brain.firstname,
                    lastname=brain.lastname,
                    nl_language=brain.nl_language,
                    organization=brain.organization,
                )
            )

        # External subscribers
        external_source_name = self.context.getSubscriberSource()
        if external_source_name != "default":
            try:
                external_source = getUtility(ISubscriberSource, name=external_source_name)
            except ComponentLookupError:
                pass

            for subscriber in external_source.getSubscribers(self.context):
                subscriber["source"] = external_source_name
                subscribers.append(subscriber)

        return subscribers

    def can_delete(self):
        meth = self.request.get("REQUEST_METHOD")
        delete_button = self.request.get("delete")
        return meth.lower() == "post" and delete_button

    def delete(self):
        """delete all the selected subscribers"""
        msg_manager = IStatusMessage(self.request)
        ids = self.request.get("subscriber_ids", [])
        if not ids:
            msg = _("No subscriber selected!")
            msg_manager.addStatusMessage(msg, type="error")
            return False
        existing = self.context.objectIds()
        # avoid wrong id to be submitted
        to_remove = [i for i in ids if i in existing]
        self.context.manage_delObjects(to_remove)
        msg = _("subscriber/s deleted successfully")
        msg_manager.addStatusMessage(msg, type="info")
        return True

    def addTokenToUrl(self, url):
        return addTokenToUrl(url)


class UploadCSV(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def create_subscribers(self, csv_data=None):
        """Create newsletter subscribers from uploaded CSV file."""

        # Do nothing if no submit button was hit
        if "form.button.Import" not in self.request.form:
            return

        context = aq_inner(self.context)
        lang = context.Language()
        # plone_utils = getToolByName(self.context, 'plone_utils')
        # encoding = plone_utils.getSiteEncoding()
        existing = []
        subscribers = api.content.find(context=self.context, portal_type="ENLSubscriber")
        for subscriber in subscribers:
            existing.append(subscriber.email.lower())

        # messages = IStatusMessage(self.request)
        success = []
        fail = []

        # Show error if no file was specified
        filename = self.request.form.get("csv_upload", None)
        if not filename:
            msg = _("No file specified.")
            IStatusMessage(self.request).addStatusMessage(msg, type="error")
            return self.request.response.redirect(context.absolute_url() + "/@@upload_csv")

        # Show error if no data has been provided in the file
        reader = UnicodeReader(filename)
        header = reader.next()
        if header != [x for x in CSV_HEADER]:
            logger.info("Got header %s\n Expected:%s" % (header, CSV_HEADER))
            msg = _("Wrong specification of the CSV file. " + "Please correct it and retry.")
            IStatusMessage(self.request).addStatusMessage(msg, type="error")
            return self.request.response.redirect(context.absolute_url() + "/@@upload_csv")

        for subscriber in reader:
            # Check the length of the line
            if len(subscriber) != 7:
                msg = _(
                    "The number of items in the line is not correct. \
                        It should be 7. Check your CSV file."
                )
                fail.append({"failure": msg})
            else:
                salutation = subscriber[0]
                name_prefix = subscriber[1]
                firstname = subscriber[2]
                lastname = subscriber[3]
                nl_language = subscriber[4]
                email = subscriber[5].lower()
                organization = subscriber[6]
                id = normalize_id(email)
                if email in existing:
                    # If subscriber with email exists, update user info
                    sub = api.content.find(context=self.context, email=email)
                    if len(sub) > 1:
                        msg = _(
                            "More than one subscriber with this email "
                            "address existed, subscriber info was NOT "
                            "updated. Check manually!"
                        )
                        fail.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            "nl_language": nl_language,
                            "email": email,
                            "organization": organization,
                            "failure": msg,
                        })
                    else:
                        sub = sub[0].getObject()
                        sub.email = email
                        sub.name_prefix = name_prefix
                        sub.firstname = firstname
                        sub.lastname = lastname
                        sub.nl_language = nl_language
                        sub.organization = organization
                        sub.salutation = salutation
                        sub.title = email + " - " + " ".join([lastname, firstname])
                        sub.reindexObject()
                        msg = _("Email existed, updated subscriber.")
                        success.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            "nl_language": nl_language,
                            "email": email,
                            "organization": organization,
                            "success": msg,
                        })

                else:
                    # If it doesn't exist, create subscriber
                    title = email + " - " + " ".join([lastname, firstname])
                    try:
                        self.context.invokeFactory(
                            "ENLSubscriber",
                            id=id,
                            title=title,
                            description="",
                            language=lang,
                        )
                        sub = context[id]
                        sub.email = email
                        sub.name_prefix = name_prefix
                        sub.firstname = firstname
                        sub.lastname = lastname
                        sub.nl_language = nl_language
                        sub.organization = organization
                        sub.salutation = salutation
                        sub.reindexObject()
                        # update existing
                        existing.append(email)
                        msg = _("Subscriber created.")
                        success.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            "nl_language": nl_language,
                            "email": email,
                            "organization": organization,
                            "success": msg,
                        })

                    except Exception as e:
                        fail.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            "nl_language": nl_language,
                            "email": email,
                            "organization": organization,
                            "failure": "An error occured while creating this subscriber: %s"
                            % str(e),
                        })

        return {"success": success, "fail": fail}


class DownloadCSV(BrowserView):
    def __call__(self):
        """Returns a CSV file with all newsletter subscribers."""
        context = aq_inner(self.context)
        ctool = getToolByName(context, "portal_catalog")

        # Create CSV file
        filename = tempfile.mktemp()
        file = open(filename, "wb")
        csvWriter = UnicodeWriter(
            file, {"delimiter": ",", "quotechar": '"', "quoting": csv.QUOTE_MINIMAL}
        )
        csvWriter.writerow([x for x in CSV_HEADER])
        for subscriber in ctool(
            portal_type="ENLSubscriber",
            path="/".join(self.context.getPhysicalPath()),
            sort_on="email",
        ):
            obj = subscriber.getObject()
            csvWriter.writerow([
                obj.salutation,
                obj.name_prefix,
                obj.firstname,
                obj.lastname,
                obj.nl_language,
                obj.email,
                obj.organization,
            ])
        file.close()
        data = open(filename).read()

        # Create response
        response = context.REQUEST.response
        response.addHeader(
            "Content-Disposition", "attachment; filename=easynewsletter-subscribers.csv"
        )
        response.addHeader("Content-Type", "text/csv")
        response.addHeader("Content-Length", "%d" % len(data))
        response.addHeader("Pragma", "no-cache")
        response.addHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0, public")
        response.addHeader("Expires", "0")

        # Return CSV data
        return data



from Products.EasyNewsletter.config import ENL_EDITHELPER_TYPES
from Products.Five.browser import BrowserView


class EditHelperView(BrowserView):
    """ """

    def enable(self):
        return self.context.portal_type in ENL_EDITHELPER_TYPES



from AccessControl.SecurityManagement import newSecurityManager
from plone import api, protect
from zope.interface import alsoProvides

from Products.EasyNewsletter import _
from Products.EasyNewsletter.content.newsletter import INewsletter
from Products.EasyNewsletter.content.newsletter_subscriber import INewsletterSubscriber
from Products.Five.browser import BrowserView


class NewsletterUnsubscribe(BrowserView):
    def __call__(self):
        """ """
        alsoProvides(self.request, protect.interfaces.IDisableCSRFProtection)
        uid = self.request.get("subscriber")

        newsletter = self.context
        if not INewsletter.providedBy(newsletter):
            api.portal.show_message(
                message=_("Please use the correct unsubscribe url!"),
                request=self.request,
                type="error",
            )
            return self.request.response.redirect(
                api.portal.get_navigation_root(self).absolute_url()
            )

        # We do the deletion as the owner of the newsletter object
        # so that this is possible without login.
        owner = newsletter.getWrappedOwner()
        newSecurityManager(newsletter, owner)
        subscriber = api.content.get(UID=uid)
        if subscriber is None or not INewsletterSubscriber.providedBy(subscriber):
            api.portal.show_message(
                message=_("An error occured"),
                request=self.request,
                type="error",
            )
        else:
            del newsletter[subscriber.id]
            api.portal.show_message(
                message=_("You have been unsubscribed."),
                request=self.request,
                type="info",
            )

        return self.request.response.redirect(api.portal.get_navigation_root(self).absolute_url())



# -*- coding: utf-8 -*-
# flake8: noqa
try:
    return context.portal_interface.objectImplements(context, "OFS.OrderSupport.IOrderedContainer")
except:
    return context.portal_interface.objectImplements(
        context, "OFS.OrderSupport.z2IOrderedContainer"
    )



from . import logger
from .base import reload_gs_profile

# from plone import api


def upgrade(setup_tool=None):
    """ """
    logger.info(
        "Running upgrade (Python): upgrade portal_types settings to easynewsletter actions categoryname having a prefix"
    )
    reload_gs_profile(setup_tool)



from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from zope.component import adapter
from zope.interface import Interface, implementer, provider

from Products.EasyNewsletter import _


class ICollectionAsNewsletterAggregationSourceMarker(Interface):
    pass


@provider(IFormFieldProvider)
class ICollectionAsNewsletterAggregationSource(model.Schema):
    """ """

    model.fieldset(
        "settings",
        label=_("Settings"),
        fields=["aggregation_template"],
    )

    aggregation_template = schema.Choice(
        title=_("Newsletter aggregation template"),
        description=_(
            'The <a href="https://productseasynewsletter.readthedocs.io/en/latest/#aggregation-templates">aggregation template</a> used by the Newsletter to render Collection items for the Newsletter.'
        ),
        vocabulary="Products.EasyNewsletter.AggregationTemplates",
        # defaultFactory=get_default_aggregation_template,
        required=True,
    )


@implementer(ICollectionAsNewsletterAggregationSource)
@adapter(ICollectionAsNewsletterAggregationSourceMarker)
class CollectionAsNewsletterAggregationSource:
    def __init__(self, context):
        self.context = context

    @property
    def aggregation_template(self):
        if hasattr(self.context, "aggregation_template"):
            return self.context.aggregation_template
        return None

    @aggregation_template.setter
    def aggregation_template(self, value):
        self.context.aggregation_template = value



from Products.EasyNewsletter import EasyNewsletterMessageFactory as _  # noqa
from zope import schema
from zope.interface import Interface


class IENLSettings(Interface):
    """ """

    allowed_content_aggregation_types = schema.List(
        required=False,
        title=_("Allowed content aggregation types"),
        description=_(
            "Content types which will be visible in the Content"
            " aggregation sources reference field."
        ),
        value_type=schema.TextLine(title=_("Content type")),
    )



from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import Interface

from Products.EasyNewsletter.browser.controlpanel import IENLSettings


class IENLUtils(Interface):
    """ENL utils methods"""


class ENLUtils:
    """ENL utils methods"""

    def get_allowed_content_aggregation_types(self):
        """return a tuple of allowed content types for content aggregation"""
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IENLSettings, prefix="Products.EasyNewsletter")
        return settings.allowed_content_aggregation_types



import os
import subprocess

package_name = "Products.EasyNewsletter"
# domain = package_name
package_domain = "Products.EasyNewsletter"
locale_path = os.path.dirname(os.path.realpath(__file__))
target_path = os.path.abspath(os.path.join(locale_path, os.pardir))
i18ndude = "./bin/i18ndude"


def locale_folder_setup(domain=None):
    languages = [
        d
        for d in os.listdir(locale_path)
        if os.path.isdir(os.path.join(locale_path, d)) and d != "__pycache__"
    ]
    for lang in languages:
        lc_message_dir_path = os.path.join(locale_path, lang, "LC_MESSAGES")
        if not os.path.isdir(lc_message_dir_path):
            os.mkdir(lc_message_dir_path)
        domain_po_file_path = os.path.join(locale_path, lang, "LC_MESSAGES", domain + ".po")
        if not os.path.isfile(domain_po_file_path):
            cmd = "msginit --locale={0} --input={2}/manual.pot --output={2}/{0}/LC_MESSAGES/{1}.po".format(  # NOQA: E501
                lang, domain, locale_path
            )
            subprocess.call(cmd, shell=True)


def _rebuild(domain=None, target_path=None):
    print(f"rebuild-pot domain: {domain} target_path: {target_path}")
    cmd = f"{i18ndude} rebuild-pot --no-wrap --pot {locale_path}/{domain}.pot --create {domain} {target_path} --merge {locale_path}/manual.pot"
    if domain == package_domain:
        cmd += f" --merge {locale_path}/manual.pot"
    subprocess.call(cmd, shell=True)


# def _filter(domain=None, filterdomain=u'plone'):
#     cmd = '{0} filter {1}.pot {2}/{3}.pot'.format(
#         i18ndude,
#         filterdomain,
#         locale_path,
#         domain,
#     )
#     subprocess.call(
#         cmd,
#         shell=True,
#     )


def _sync(domain=None):
    print(f"sync domain: {domain}")
    cmd = (
        f"{i18ndude} sync --pot {locale_path}/{domain}.pot {locale_path}/**/LC_MESSAGES/{domain}.po"
    )
    subprocess.call(cmd, shell=True)


def update_locale():

    locale_folder_setup(domain=package_domain)
    _rebuild(domain=package_domain, target_path=target_path)
    _sync(domain=package_domain)

    # also build locales für domain: plone
    locale_folder_setup(domain="plone")
    _rebuild(domain="plone", target_path=os.path.join(target_path, "profiles"))
    # _filter(domain=package_domain, filterdomain='plone')
    _sync(domain="plone")



import logging

from plone import api
from plone.protect.utils import addTokenToUrl
from zope.component import getUtility
from zope.interface.interfaces import ComponentLookupError

from Products.EasyNewsletter import _, config
from Products.EasyNewsletter.interfaces import ISubscriberSource
from Products.Five.browser import BrowserView

log = logging.getLogger("Products.EasyNewsletter")


class NewsletterSubscribers(BrowserView):
    # TODO: we should move these indexes from FieldIndex to ZCTextIndex
    # see setuphandlers.py for indexes creation
    searchable_params = ("SearchableText",)

    def __call__(self):
        if self.can_delete():
            self.delete()
        return self.index()

    def subscribers(self):
        query = dict(
            portal_type="Newsletter Subscriber",
            context=self.context,
            sort_on="email",
        )
        form = self.request.form
        for k in self.searchable_params:
            if form.get(k):
                if k == "SearchableText":
                    searchterm = form.get(k)
                    if not searchterm.endswith("*"):
                        searchterm += "*"
                    query[k] = searchterm
                else:
                    query[k] = form.get(k)
        subscribers = list()

        # Plone subscribers
        for brain in api.content.find(**query):
            if brain.salutation:
                salutation = config.SALUTATION.get(brain.salutation, "")
            else:
                salutation = ""
            subscribers.append(
                dict(
                    id=brain.getId,
                    source="plone",
                    deletable=True,
                    email=brain.email,
                    getURL=brain.getURL(),
                    salutation=salutation,
                    name_prefix=brain.name_prefix,
                    firstname=brain.firstname,
                    lastname=brain.lastname,
                    nl_language=brain.nl_language,
                    organization=brain.organization,
                )
            )

        # External subscribers
        ext_subcriber_source = self.context.get("subscriber_source")
        if ext_subcriber_source:
            if ext_subcriber_source != "default":
                try:
                    external_source = getUtility(ISubscriberSource, name=ext_subcriber_source)
                except ComponentLookupError:
                    log.warn(
                        _(
                            "label_ext_subcriber_source_failed",
                            default="External subscriber lookup failed",
                        )
                    )
                else:
                    for subscriber in external_source.getSubscribers(self.context):
                        subscriber["source"] = ext_subcriber_source
                        subscribers.append(subscriber)

        return subscribers

    def can_delete(self):
        meth = self.request.get("REQUEST_METHOD")
        delete_button = self.request.get("delete")
        return meth.lower() == "post" and delete_button

    def delete(self):
        """delete all the selected subscribers"""
        ids = self.request.get("subscriber_ids", [])
        if not ids:
            msg = _("No subscriber selected!")
            api.portal.show_message(msg, request=self.request, type="error")
            return False
        existing = self.context.objectIds()
        # avoid wrong id to be submitted
        to_remove = [i for i in ids if i in existing]
        self.context.manage_delObjects(to_remove)
        msg = _("subscriber/s deleted successfully")
        api.portal.show_message(msg, request=self.request, type="info")
        return True

    def addTokenToUrl(self, url):
        return addTokenToUrl(url)



import unittest

from plone import api
from plone.api.exc import InvalidParameterError
from plone.app.testing import TEST_USER_ID, setRoles
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject, queryUtility

from Products.EasyNewsletter.content.newsletter import INewsletter
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class NewsletterIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_ct_newsletter_schema(self):
        fti = queryUtility(IDexterityFTI, name="Newsletter")
        schema = fti.lookupSchema()
        self.assertEqual(INewsletter, schema)

    def test_ct_newsletter_fti(self):
        fti = queryUtility(IDexterityFTI, name="Newsletter")
        self.assertTrue(fti)

    def test_ct_newsletter_factory(self):
        fti = queryUtility(IDexterityFTI, name="Newsletter")
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            INewsletter.providedBy(obj),
            f"INewsletter not provided by {obj}!",
        )

    def test_ct_newsletter_adding(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        obj = api.content.create(
            container=self.portal,
            type="Newsletter",
            id="newsletter",
        )

        self.assertTrue(
            INewsletter.providedBy(obj),
            f"INewsletter not provided by {obj.id}!",
        )

        parent = obj.__parent__
        self.assertIn("newsletter", parent.objectIds())

        # check that deleting the object works too
        api.content.delete(obj=obj)
        self.assertNotIn("newsletter", parent.objectIds())

    def test_ct_newsletter_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="Newsletter")
        self.assertTrue(fti.global_allow, f"{fti.id} is not globally addable!")

    def test_ct_newsletter_filter_content_type_true(self):
        setRoles(self.portal, TEST_USER_ID, ["Contributor"])
        fti = queryUtility(IDexterityFTI, name="Newsletter")
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            fti.id,
            self.portal,
            "newsletter_id",
            title="Newsletter container",
        )
        parent = self.portal[parent_id]
        with self.assertRaises(InvalidParameterError):
            api.content.create(
                container=parent,
                type="Document",
                title="My Content",
            )



import unittest

from plone.app.testing import TEST_USER_ID, setRoles
from plone.behavior.interfaces import IBehavior
from zope.component import getUtility

from Products.EasyNewsletter.behaviors.plone_user_group_recipients import (
    IPloneUserGroupRecipientsMarker,
)
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class PloneUserGroupRecipientsIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_behavior_plone_user_group_recipients(self):
        behavior = getUtility(IBehavior, "Products.EasyNewsletter.plone_user_group_recipients")
        self.assertEqual(
            behavior.marker,
            IPloneUserGroupRecipientsMarker,
        )



import os
import unittest

import transaction as zt
from App.Common import package_home
from plone import api
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, setRoles
from plone.app.textfield import RichTextValue
from zExceptions import Forbidden
from zope.component import getGlobalSiteManager, getMultiAdapter, getSiteManager, provideHandler

from Products.CMFPlone.tests.utils import MockMailHost
from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter.interfaces import IBeforePersonalizationEvent
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING
from Products.EasyNewsletter.tests.base import parsed_attachments_from_msg, parsed_payloads_from_msg
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.MailHost.interfaces import IMailHost

GLOBALS = globals()
TESTS_HOME = package_home(GLOBALS)


def dummy_image(imgname="img1.jpg"):
    from plone.namedfile.file import NamedBlobImage

    filename = os.path.join(os.path.dirname(__file__), imgname)
    with open(filename, "rb") as f:
        return NamedBlobImage(data=f.read(), filename=filename)


class EasyNewsletterTests(unittest.TestCase):
    # layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.mail_settings = get_portal_mail_settings()
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Folder", "test-folder")
        self.folder = self.portal["test-folder"]
        self.portal = self.layer["portal"]
        self.folder.invokeFactory("Newsletter", "newsletter")
        self.newsletter = self.folder.newsletter
        self.newsletter.sender_email = "newsletter@acme.com"
        self.newsletter.sender_name = "ACME newsletter"
        self.newsletter.test_email = "test@acme.com"
        prologue_output = self.newsletter.default_prologue.output
        self.default_prologue = RichTextValue(
            raw=prologue_output,
            mimeType="text/html",
            outputMimeType="text/x-plone-outputfilters-html",
        )
        epilogue_output = self.newsletter.default_epilogue.output
        self.default_epilogue = RichTextValue(
            raw=epilogue_output,
            mimeType="text/html",
            outputMimeType="text/x-plone-outputfilters-html",
        )

        # Set up a mock mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        # We need to fake a valid mail setup
        # self.mail_settings.email_from_address = u'portal@plone.test'
        self.mailhost = self.portal.MailHost
        # image1 for image testing
        self.folder.invokeFactory("Image", "image1")
        self.image1 = self.folder.image1
        image1 = self.folder["image1"]
        image1.title = "My Image 1"
        image1.description = "This is my image."
        image1.image = dummy_image()
        self.image1 = image1
        self.image1_tag = api.content.get_view("images", self.image1, self.request).tag(
            "image", "thumb"
        )

        # image2 for image testing
        self.folder.invokeFactory("Image", "image2")
        self.image2 = self.folder.image2
        image2 = self.folder["image2"]
        image2.title = "My Image 2"
        image2.description = "This is my image."
        image2.image = dummy_image(imgname="img2.jpg")
        self.image2 = image2
        image2_images = api.content.get_view("images", self.image2, self.request)
        self.image2_scale = image2_images.scale("image", "thumb")
        self.image2_url = self.image2_scale.url
        self.image2_uid = self.image2_scale.uid

        # image3 for image testing
        self.folder.invokeFactory("Image", "image3")
        self.image3 = self.folder.image3
        image3 = self.folder["image3"]
        image3.title = "My Image 3"
        image3.description = "This is my image."
        image3.image = dummy_image(imgname="img3.jpg")
        self.image3 = image3
        image3_images = api.content.get_view("images", self.image3, self.request)
        self.image3_scale = image3_images.scale("image", "thumb")
        self.image3_url = self.image3_scale.url
        self.image3_uid = self.image3_scale.uid

    def send_sample_message(self, body):
        self.assertSequenceEqual(self.mailhost.messages, [])
        self.issue = api.content.create(
            type="Newsletter Issue",
            id="issue",
            title="with image",
            container=self.newsletter,
        )
        self.issue.text = RichTextValue(
            raw=body,
            mimeType="text/html",
            outputMimeType="text/x-plone-outputfilters-html",
        )
        self.issue.prologue = self.default_prologue
        self.issue.epilogue = self.default_epilogue
        self.issue.output_template = "output_default"

        self.portal.REQUEST.form.update({
            "sender_name": self.newsletter.sender_name,
            "sender_email": self.newsletter.sender_email,
            "test_receiver": self.newsletter.test_email,
            "subject": self.issue.title,
            "test": "submit",
        })
        self.portal.REQUEST["REQUEST_METHOD"] = "POST"
        zt.commit()
        view = getMultiAdapter((self.issue, self.portal.REQUEST), name="send-issue")

        view()

        self.assertEqual(len(self.mailhost.messages), 1)
        self.assertTrue(self.mailhost.messages[0])
        return safe_unicode(self.mailhost.messages[0])

    def test_issue_send_test(self):
        self.issue = api.content.create(
            type="Newsletter Issue",
            id="issue",
            title="Issue 1",
            container=self.newsletter,
        )
        self.issue.title = (
            "This is a very long newsletter issue title with special "
            "characters such as äüö. Will this really work?"
        )
        # self.issue.text = RichTextValue(
        #     raw=u'nothing',
        #     mimeType="text/html",
        #     outputMimeType="text/x-plone-outputfilters-html",
        #     encoding="utf-8",
        # )
        self.issue.output_template = "output_default"
        zt.commit()

        self.portal.REQUEST.form.update({
            "sender_name": self.newsletter.sender_name,
            "sender_email": self.newsletter.sender_email,
            "test_receiver": self.newsletter.test_email,
            "subject": self.issue.title,
            "test": "submit",
        })
        self.portal.REQUEST["REQUEST_METHOD"] = "POST"
        view = getMultiAdapter((self.issue, self.portal.REQUEST), name="send-issue")
        view()

        self.assertEqual(len(self.mailhost.messages), 1)
        self.assertTrue(self.mailhost.messages[0])
        msg = safe_unicode(self.mailhost.messages[0])
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn("Test Member", parsed_payloads["to"])
        self.assertIn("<test@acme.com>", parsed_payloads["to"])
        self.assertIn("<newsletter@acme.com>", parsed_payloads["from"])
        self.assertIn("ACME newsletter", parsed_payloads["from"])

    def test_send_test_personalization(self):
        # with all infos
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            salutation="ms",
            title="jane@example.com",
            firstname="Jane",
            lastname="Doe",
            email="jane@example.com",
        )
        # without salutation
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            title="john@example.com",
            firstname="John",
            lastname="Doe",
            email="john@example.com",
        )
        # without firstname
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            title="max@example.com",
            lastname="Mustermann",
            email="max@example.com",
        )
        # without lastname
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            title="maxima@example.com",
            firstname="Maxima",
            email="maxima@example.com",
        )
        # without firstname and lastname
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            title="leo@example.com",
            email="leo@example.com",
        )
        body = "Some body content, in the issue."
        self.issue = api.content.create(
            type="Newsletter Issue",
            id="issue",
            title=(
                "This is a very long newsletter issue title with special "
                "characters such as äüö. Will this really work?"
            ),
            container=self.newsletter,
            prologue=self.default_prologue,
            epilogue=self.default_epilogue,
            text=RichTextValue(
                raw=body,
                mimeType="text/html",
                outputMimeType="text/x-plone-outputfilters-html",
                encoding="utf-8",
            ),
            output_template="output_default",
        )
        zt.commit()
        self.portal.REQUEST.form.update({
            "sender_name": self.newsletter.sender_name,
            "sender_email": self.newsletter.sender_email,
            "test_receiver": self.newsletter.test_email,
            "subject": self.issue.title,
        })
        self.portal.REQUEST["REQUEST_METHOD"] = "POST"
        view = getMultiAdapter((self.issue, self.portal.REQUEST), name="send-issue")
        view()

        self.assertEqual(len(self.mailhost.messages), 5)
        self.assertTrue(self.mailhost.messages[0])
        self.assertTrue(self.mailhost.messages[1])

        msg1 = safe_unicode(self.mailhost.messages[0])
        parsed_payloads1 = parsed_payloads_from_msg(msg1)
        self.assertIn("Jane Doe", parsed_payloads1["to"])
        self.assertIn("<jane@example.com>", parsed_payloads1["to"])
        self.assertIn("Dear Ms. Jane Doe", safe_unicode(parsed_payloads1["text/html"]))

        msg2 = safe_unicode(self.mailhost.messages[1])
        parsed_payloads2 = parsed_payloads_from_msg(msg2)
        self.assertIn("John Doe", parsed_payloads2["to"])
        self.assertIn("Dear John Doe", safe_unicode(parsed_payloads2["text/html"]))

        msg3 = safe_unicode(self.mailhost.messages[2])
        parsed_payloads3 = parsed_payloads_from_msg(msg3)
        self.assertIn("Mustermann", parsed_payloads3["to"])
        self.assertIn("<max@example.com>", parsed_payloads3["to"])
        self.assertIn("Dear Mustermann", safe_unicode(parsed_payloads3["text/html"]))

        msg4 = safe_unicode(self.mailhost.messages[3])
        parsed_payloads4 = parsed_payloads_from_msg(msg4)
        self.assertIn("Maxima", parsed_payloads4["to"])
        self.assertIn("<maxima@example.com>", parsed_payloads4["to"])
        self.assertIn("Dear Maxima", safe_unicode(parsed_payloads4["text/html"]))

        msg5 = safe_unicode(self.mailhost.messages[4])
        parsed_payloads5 = parsed_payloads_from_msg(msg5)
        self.assertIn("leo@example.com", parsed_payloads5["to"])
        self.assertIn("Sir or Madam", safe_unicode(parsed_payloads5["text/html"]))

    def test_before_the_personalization_filter(self):
        def _personalize(event):
            edc = event.data["context"]
            event.data["html"] = event.data["html"].replace("PHP", "Python")
            firstname = edc["receiver"].get("firstname")
            lastname = edc["receiver"].get("lastname")
            if not firstname and not lastname:
                edc["SUBSCRIBER_SALUTATION"] = "Dear {0}".format(edc["receiver"]["email"])

        provideHandler(_personalize, [IBeforePersonalizationEvent])
        try:
            # with all infos
            api.content.create(
                type="Newsletter Subscriber",
                container=self.newsletter,
                salutation="ms",
                title="jane@example.com",
                firstname="Jane",
                lastname="Doe",
                email="jane@example.com",
            )
            # without firstname and lastname
            api.content.create(
                type="Newsletter Subscriber",
                container=self.newsletter,
                salutation="mr",
                title="john@example.com",
                email="john@example.com",
            )

            self.issue = api.content.create(
                type="Newsletter Issue",
                id="issue",
                title="Issue 1",
                container=self.newsletter,
            )
            self.issue.title = (
                "This is a very long newsletter issue title with special "
                "characters such as äüö. Will this really work?"
            )
            body = """
                <h1>PHP is cool</h1>
                {{SUBSCRIBER_SALUTATION}}
                """
            self.issue.text = RichTextValue(
                raw=body,
                mimeType="text/html",
                outputMimeType="text/x-plone-outputfilters-html",
                encoding="utf-8",
            )
            self.issue.prologue = self.default_prologue
            self.issue.epilogue = self.default_epilogue
            self.issue.output_template = "output_default"
            self.portal.REQUEST.form.update({
                "sender_name": self.newsletter.sender_name,
                "sender_email": self.newsletter.sender_email,
                "test_receiver": self.newsletter.test_email,
                "subject": self.issue.title,
            })
            self.portal.REQUEST["REQUEST_METHOD"] = "POST"
            zt.commit()
            # clearEvents()  # noqa
            view = getMultiAdapter((self.issue, self.portal.REQUEST), name="send-issue")
            view()

            # pers_events = getEvents(IBeforePersonalizationFilter)
            # print(pers_events)
            self.assertEqual(len(self.mailhost.messages), 2)
            msg1 = safe_unicode(self.mailhost.messages[0])
            parsed_payloads1 = parsed_payloads_from_msg(msg1)
            self.assertIn("Jane Doe", parsed_payloads1["to"])
            self.assertIn("<jane@example.com>", parsed_payloads1["to"])
            self.assertIn("Dear Ms. Jane Doe", safe_unicode(parsed_payloads1["text/html"]))

            msg2 = safe_unicode(self.mailhost.messages[1])
            parsed_payloads2 = parsed_payloads_from_msg(msg2)
            self.assertIn("john@example.com", parsed_payloads2["to"])
            self.assertIn("Dear john@example.com", safe_unicode(parsed_payloads2["text/html"]))
        finally:
            getGlobalSiteManager().unregisterHandler(_personalize, [IBeforePersonalizationEvent])

    def test_send_test_issue_with_image(self):
        body = f'<img src="{self.image1.absolute_url_path()}"/>'
        msg = self.send_sample_message(body)
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn('src="cid:image', safe_unicode(parsed_payloads["text/html"]))
        self.assertIn("Content-ID: <image", safe_unicode(msg))
        # mimetype is not detected by python-emails because of missing file extension,
        # until https://github.com/lavr/python-emails/issues/163 is fixed.
        self.assertIn("Content-Type: application/unknown;", safe_unicode(msg))
        # self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))

    def test_send_test_issue_with_scale_image(self):
        body = f'<img src="{self.image1.absolute_url_path()}/@@images/image/thumb"/>'
        # trigger scale generation:
        image_scales_url = f"{self.image1.absolute_url_path()}/@@images"
        scales = self.portal.restrictedTraverse(image_scales_url)
        scale_view = scales.scale(fieldname="image", scale="thumb")
        scale_view()
        # scale_view.index_html()
        zt.commit()

        msg = self.send_sample_message(body)
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn('src="cid:thumb', safe_unicode(parsed_payloads["text/html"]))
        self.assertIn("Content-ID: <thumb", safe_unicode(msg))
        # mimetype is not detected by python-emails because of missing file extension,
        # until https://github.com/lavr/python-emails/issues/163 is fixed.
        self.assertIn("Content-Type: application/unknown;", safe_unicode(msg))
        # self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))

    def test_send_test_issue_with_hashed_scale_image(self):
        # trigger scale generation:
        image_scales_url = f"{self.image2.absolute_url_path()}/@@images"
        scales = self.portal.restrictedTraverse(image_scales_url)
        scale_view = scales.scale(fieldname="image", scale="thumb")
        scale_view()
        # scale_view.index_html()
        zt.commit()

        # create img tag with hasged image:
        # @@images/71d2fe96-e930-4265-9cd8-e3d4123d75f5.jpeg
        body = f'<img src="{scale_view.url}"/>'

        msg = self.send_sample_message(body)
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn(
            'src="cid:{0}'.format(scale_view.url.split("/")[-1]),
            safe_unicode(parsed_payloads["text/html"]),
        )
        self.assertIn("Content-ID: <{0}>".format(scale_view.url.split("/")[-1]), safe_unicode(msg))
        self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))

    def test_send_test_issue_with_resolveuid_image(self):
        body = f'<img src="../../resolveuid/{self.image1.UID()}"/>'
        msg = self.send_sample_message(body)
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertNotIn("resolveuid", safe_unicode(parsed_payloads["text/html"]))
        self.assertIn('src="cid:image', safe_unicode(parsed_payloads["text/html"]))
        self.assertIn("Content-ID: <image", msg)
        try:
            self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))
        except AssertionError:
            # Plone < 6
            # mimetype is not detected by python-emails because of missing file extension,
            # until https://github.com/lavr/python-emails/issues/163 is fixed.
            self.assertIn("Content-Type: application/unknown;", safe_unicode(msg))

    def test_send_test_issue_with_resolveuid_scale_image(self):
        path = "image/thumb"
        stack = path.split("/")

        # trigger scale generation:
        image_scales_url = f"{self.image1.absolute_url_path()}/@@images"
        scales = self.portal.restrictedTraverse(image_scales_url)
        scale_view = scales.scale(fieldname=stack[0], scale=stack[1])
        image_scale = scale_view()
        body = f'<img src="{image_scale.absolute_url()}"/>'
        zt.commit()
        msg = self.send_sample_message(body)
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertNotIn("resolveuid", safe_unicode(parsed_payloads["text/html"]))
        self.assertIn(
            f'src="cid:{image_scale.__name__}"',
            safe_unicode(parsed_payloads["text/html"]),
        )
        self.assertIn(f"Content-ID: <{image_scale.__name__}>", msg)
        self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))

    def test_send_test_issue_with_banner_image(self):
        self.newsletter.banner = dummy_image("img1.jpg")
        zt.commit()
        msg = self.send_sample_message("")
        self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))
        self.assertNotIn("Content-Type: application/unknown;", safe_unicode(msg))
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn('src="cid:banner', safe_unicode(parsed_payloads["text/html"]))

    def test_mailonly_filter_in_issue_public_view(self):
        self.issue = api.content.create(
            type="Newsletter Issue",
            id="issue",
            title="Issue 1",
            container=self.newsletter,
        )
        self.issue.title = "Test Newsletter Issue"
        body = (
            '<h1>This is the newsletter body!</h1><div class="mailonly">'
            "This test should only visible in mails not in public view!</div>"
        )
        self.issue.text = RichTextValue(
            raw=body,
            mimeType="text/html",
            outputMimeType="text/x-plone-outputfilters-html",
            encoding="utf-8",
        )
        self.issue.prologue = self.default_prologue
        self.issue.epilogue = self.default_epilogue
        self.issue.output_template = "output_default"
        zt.commit()
        view = getMultiAdapter((self.issue, self.portal.REQUEST), name="get-public-body")
        view_result = view()

        self.assertTrue(
            "mailonly" not in safe_unicode(view_result),
            "get-public-body view contains mailonly elements, this should filtert out!",
        )

    def test_send_test_issue_with_multiple_images(self):
        body = f"""<h1>Headline</h1><br />
        {self.image1_tag}
        <img src="{self.image2_url}" />
        <img src="{self.image3_url}" />
        """
        msg = self.send_sample_message(body)
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn(
            f'src="cid:{self.image3_uid}',
            safe_unicode(parsed_payloads["text/html"]),
        )
        self.assertIn(f"Content-ID: <{self.image2_uid}.jpeg>", safe_unicode(msg))
        self.assertIn(f"Content-ID: <{self.image3_uid}.jpeg>", safe_unicode(msg))
        self.assertIn("Content-Type: image/jpeg;", safe_unicode(msg))

        attachments = parsed_attachments_from_msg(msg)
        self.assertIn(f"{self.image2_uid}.jpeg", attachments)
        self.assertIn(f"{self.image3_uid}.jpeg", attachments)

    def test_permission(self):
        setRoles(self.portal, TEST_USER_ID, ["Editor"])
        self.portal.REQUEST.set("ACTUAL_URL", "http://nohost")
        self.issue = api.content.create(
            type="Newsletter Issue",
            id="issue",
            title="Issue 1",
            container=self.newsletter,
        )
        self.issue.title = "Test Newsletter Issue"
        body = "<h1>This is the newsletter body!"
        self.issue.text = RichTextValue(
            raw=body,
            mimeType="text/html",
            outputMimeType="text/x-plone-outputfilters-html",
            encoding="utf-8",
        )
        self.issue.prologue = self.default_prologue
        self.issue.epilogue = self.default_epilogue
        self.issue.output_template = "output_default"

        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="newsletter-drafts")
        view_result = view()
        self.assertIn("test-folder/newsletter/issue", view_result)

        view = self.newsletter.restrictedTraverse("issue/send-issue-form")
        view_result = view()

        self.assertIn("Test Newsletter", view_result)

        # Editor is not allowed to call the one-step send-issue meant for cron
        # XXX
        # with self.assertRaises(Unauthorized):
        #     self.newsletter.restrictedTraverse("issue/send-issue-immediately")

        # check for postonly
        view = self.newsletter.restrictedTraverse("issue/send-issue")
        with self.assertRaises(Forbidden):
            view()


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



import unittest

from plone.app.testing import TEST_USER_ID, setRoles
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory, IVocabularyTokenized

from Products.EasyNewsletter import _
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class SalutationsIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_vocab_salutations(self):
        vocab_name = "Products.EasyNewsletter.Salutations"
        factory = getUtility(IVocabularyFactory, vocab_name)
        self.assertTrue(IVocabularyFactory.providedBy(factory))

        vocabulary = factory(self.portal)
        self.assertTrue(IVocabularyTokenized.providedBy(vocabulary))
        self.assertEqual(
            vocabulary.getTerm("ms").title,
            _("Ms"),
        )



import unittest

from App.Common import package_home
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, setRoles
from plone.registry.interfaces import IRegistry
from zope.component import getSiteManager, getUtility

from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.tests.utils import MockMailHost
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.MailHost.interfaces import IMailHost

GLOBALS = globals()
TESTS_HOME = package_home(GLOBALS)


class UtilsIntegrationTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.portal_url()
        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        self.mail_host = getUtility(IMailHost)
        self.registry = getUtility(IRegistry)
        reg_mail = self.registry.forInterface(IMailSchema, prefix="plone")
        reg_mail.email_from_address = "portal@plone.test"
        reg_mail.email_from_name = "Plone Master"
        reg_mail.smtp_host = "example.com"
        reg_mail.smtp_port = 25
        reg_mail.smtp_userid = "portal@plone.test"
        reg_mail.smtp_pass = "Password"

    def test_portal_mail_settings(self):
        settings = get_portal_mail_settings()
        self.assertEqual("portal@plone.test", settings.email_from_address)
        self.assertEqual("Plone Master", settings.email_from_name)
        self.assertEqual("example.com", settings.smtp_host)
        self.assertEqual(25, settings.smtp_port)
        self.assertEqual("portal@plone.test", settings.smtp_userid)
        self.assertEqual("Password", settings.smtp_pass)



import unittest

from plone import api
from plone.app.testing import TEST_USER_ID, setRoles
from zope.component import getMultiAdapter
from zope.interface.interfaces import ComponentLookupError

from Products.EasyNewsletter.testing import (
    PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING,
    PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING,
)


class ViewsIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.newsletter = api.content.create(
            container=self.portal, type="Newsletter", id="newsletter"
        )

    def test_unsubscribe_form_is_registered(self):
        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="unsubscribe-form")
        self.assertTrue(view.__name__ == "unsubscribe-form")
        # self.assertTrue(
        #     'Sample View' in view(),
        #     'Sample View is not found in unsubscribe-form'
        # )

    def test_unsubscribe_form_not_matching_interface(self):
        with self.assertRaises(ComponentLookupError):
            getMultiAdapter((self.portal, self.portal.REQUEST), name="unsubscribe-form")


class ViewsFunctionalTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



# from plone.indexer import indexer


# from Products.EasyNewsletter.interfaces import IENLSubscriber

# XXX: check if need to implement this for DX Subscriber CT's
# @indexer(IENLSubscriber)
# def SearchableText(obj):
#     user = IENLSubscriber(obj, None)
#     if user is None:
#         return None
#     searchable = ' '.join(filter(None, (obj.getName_prefix(),
#                                         obj.getFirstname(),
#                                         obj.getLastname(),
#                                         obj.getOrganization(),
#                                         obj.getEmail())))
#     return searchable



import logging

from email_validator import EmailNotValidError, validate_email
from plone import api
from plone.dexterity.interfaces import IDexterityContent
from zope.component import subscribers
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter import _
from Products.EasyNewsletter.interfaces import IReceiversMemberFilter

log = logging.getLogger("Products.EasyNewsletter")


@implementer(IVocabularyFactory)
class PloneUsers:
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        results = []
        member_properties = dict()
        users = api.user.get_users()
        for user in users:
            info = dict()
            info["id"] = user.getUserId()
            info["email"] = user.getProperty("email")
            info["fullname"] = safe_unicode(user.getProperty("fullname"))
            member_properties[info["id"]] = info

        try:
            for id, property in member_properties.items():
                try:
                    vemail = validate_email(property["email"])
                except EmailNotValidError as e:
                    log.error(
                        _('Property email: "{0}" is not an email!').format(property["email"]),
                        e,
                    )
                else:
                    results.append((id, "{0} - {1}".format(property["fullname"], vemail.email)))
        except TypeError as e:  # noqa
            log.error(
                f":get_plone_members: error in member_properties {e} \
                properties:'{member_properties.items()}'"
            )

        # run registered member filter
        for subscriber in subscribers([self], IReceiversMemberFilter):
            results = subscriber.filter(results)

        # Fix context if you are using the vocabulary in DataGridField.
        # See https://github.com/collective/collective.z3cform.datagridfield/issues/31:  # NOQA: 501
        if not IDexterityContent.providedBy(context):
            req = getRequest()
            context = req.PARENTS[0]

        # create a list of SimpleTerm items:
        terms = []
        for item in results:
            terms.append(SimpleTerm(value=item[0], token=item[0], title=item[1]))
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


PloneUsersFactory = PloneUsers()



from plone import api

from Products.Five.browser import BrowserView


class NewsletterIssueCopyAsMaster(BrowserView):
    def __call__(self):
        return self.copy_as_master()

    def copy_as_master(self):
        request = self.request
        newsletter = self.context.get_newsletter()
        master_id = "master_" + self.context.id

        master_obj = api.content.copy(
            source=self.context, target=newsletter, safe_id=True, id=master_id
        )

        request["enlwf_guard"] = True
        api.content.transition(obj=master_obj, transition="make_master")
        request["enlwf_guard"] = False

        return self.request.response.redirect(master_obj.absolute_url() + "/edit")



import logging

import jinja2
from bs4 import BeautifulSoup
from html2text import HTML2Text
from plone import api
from zope.event import notify
from zope.interface import implementer

from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter.config import PLACEHOLDERS
from Products.EasyNewsletter.interfaces import IBeforePersonalizationEvent, IIssueDataFetcher

log = logging.getLogger("Products.EasyNewsletter")


@implementer(IBeforePersonalizationEvent)
class BeforePersonalizationEvent:
    def __init__(self, data):
        self.data = data


@implementer(IIssueDataFetcher)
class DefaultDXIssueDataFetcher:
    def __init__(self, issue):
        self.issue = issue

    def __call__(self):
        """
        returns a dict of issue_data, like subject and several parts of
        the issue. This is done so, to split up the send method and
        make it more hookable.
        """
        data = {}
        request = self.issue.REQUEST
        data["subject"] = safe_unicode(request.get("subject")) or safe_unicode(self.issue.title)
        data["body_html"] = safe_unicode(self._render_output_html())
        return data

    def preview_html(self, disable_filter=False, receiver=None):
        receiver = receiver or {}
        html = self._render_output_html(preview=True)
        html = self.personalize(receiver, html)
        for placeholder in PLACEHOLDERS:
            html = html.replace("[[" + placeholder + "]]", "")
        if not disable_filter:
            soup = BeautifulSoup(html, features="lxml")
            for node in soup.findAll(True, {"class": "mailonly"}):
                node.extract()
            return soup.renderContents()
        return html

    @property
    def enl(self):
        if not hasattr(self, "_enl"):
            self._enl = self.issue.get_newsletter()
        return self._enl

    def _fullname(self, receiver):
        fullname = receiver.get("fullname") or ""
        fullname = fullname.strip()
        if not fullname:
            try:
                return self.enl.fullname_fallback
            except AttributeError:
                return "Sir or Madam"
        return fullname

    def _salutation(self, receiver):
        return receiver.get("salutation") or ""

    def _subscriber_salutation(self, receiver):
        return (
            f"{safe_unicode(self._salutation(receiver))} {safe_unicode(self._fullname(receiver))}"
        )

    def _unsubscribe_info(self, receiver):
        if "uid" not in receiver:
            return {"link": "", "text": "", "html": ""}
        try:
            unsubscribe_text = self.enl.unsubscribe_string
        except AttributeError:
            unsubscribe_text = "Click here to unsubscribe"
        unsubscribe_link = "{0}/unsubscribe?subscriber={1}".format(
            self.enl.absolute_url(), receiver["uid"]
        )
        unsubscribe_markup = (
            f"""<a href="{unsubscribe_link}" class="enl_unsubscribe_link">{unsubscribe_text}.</a>"""
        )
        return {
            "link": safe_unicode(unsubscribe_link),
            "text": safe_unicode(unsubscribe_text),
            "html": safe_unicode(unsubscribe_markup),
        }

    def _render_output_html(self, preview=False):
        """Return rendered newsletter
        with header+body+footer (raw html).
        """
        output_tmpl_id = self.issue.output_template
        issue_tmpl = self.issue.restrictedTraverse(str(output_tmpl_id))
        output_html = issue_tmpl.render()
        return output_html

    @property
    def plone_view(self):
        if hasattr(self, "_plone_view"):
            return self._plone_view
        self._plone_view = api.content.get_view(
            name="plone", context=self.enl, request=self.enl.REQUEST
        )
        return self._plone_view

    @property
    def issue_data(self):
        if hasattr(self, "_issue_data"):
            return self._issue_data
        self._issue_data = {}
        self._issue_data["title"] = self.issue.title
        self._issue_data["description"] = self.issue.description
        self._issue_data["banner_src"] = self.issue.get_image_src()
        scales = self.enl.restrictedTraverse("@@images")
        logo_src = ""
        if scales.scale("logo", scale="mini"):
            logo_src = self.enl.absolute_url() + "/@@images/logo"
        self._issue_data["logo_src"] = logo_src
        self._issue_data["date"] = self.plone_view.toLocalizedTime(
            self.issue.effective(), long_format=0
        )
        self._issue_data["month"] = self.issue.effective().month()
        self._issue_data["year"] = self.issue.effective().year()
        self._issue_data["calendar_week"] = self.issue.effective().strftime("%V")
        return self._issue_data

    def personalize(self, receiver, html):
        issue_data = self.issue_data
        data = {}
        data["html"] = html
        data["context"] = {}
        # receiver data:
        data["context"]["receiver"] = receiver
        data["context"]["language"] = self.enl.language
        data["context"]["fullname"] = self._fullname(receiver)
        data["context"]["salutation"] = self._salutation(receiver)
        data["context"]["unsubscribe_info"] = self._unsubscribe_info(receiver)
        data["context"]["unsubscribe"] = data["context"]["unsubscribe_info"]["html"]
        data["context"]["UNSUBSCRIBE"] = data["context"]["unsubscribe"]
        data["context"]["subscriber_salutation"] = self._subscriber_salutation(receiver)
        data["context"]["SUBSCRIBER_SALUTATION"] = data["context"]["subscriber_salutation"]
        # issue_data:
        data["context"]["issue_title"] = issue_data["title"]
        data["context"]["issue_description"] = issue_data["description"]
        data["context"]["banner_src"] = issue_data["banner_src"]
        data["context"]["logo_src"] = issue_data["logo_src"]
        data["context"]["date"] = issue_data["date"]
        data["context"]["month"] = issue_data["month"]
        data["context"]["year"] = issue_data["year"]
        data["context"]["calendar_week"] = issue_data["calendar_week"]

        notify(BeforePersonalizationEvent(data))
        template = jinja2.Template(safe_unicode(data["html"]))
        return template.render(**data["context"])

    def create_plaintext_message(self, text):
        """Create a plain-text-message by parsing the html
        and attaching links as endnotes
        """
        html_to_text = HTML2Text(baseurl=self.issue.absolute_url())
        html_to_text.ul_style_dash = True
        html_to_text.inline_links = False
        html_to_text.wrap_links = False
        plaintext = html_to_text.handle(text)
        return plaintext



import os
import unittest

import six
from plone import api
from plone.app.testing import TEST_USER_ID, setRoles
from zope.component import getMultiAdapter
from zope.interface.interfaces import ComponentLookupError

from Products.CMFPlone.utils import safe_encode, safe_unicode
from Products.EasyNewsletter.testing import (
    PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING,
    PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING,
)


class ViewsIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.newsletter = api.content.create(
            container=self.portal, type="Newsletter", id="newsletter"
        )

    def test_subscribers_upload_is_registered(self):
        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="subscribers-upload")
        self.assertTrue(view.__name__ == "subscribers-upload")

    def test_subscribers_upload_not_matching_interface(self):
        with self.assertRaises(ComponentLookupError):
            getMultiAdapter((self.portal, self.portal.REQUEST), name="subscribers-upload")

    def test_subscribers_upload_create_subscribers(self):
        template = """salutation,name_prefix,firstname,lastname,email,organization
Ms,PhD,Jane,Doe,jane@example.com,Doe Cörp
Mr,PhD,John,Doe,john@example.com,Doe Cörp
"""
        filename = os.path.join(os.path.dirname(__file__), "easynewsletter-subscribers.csv")
        if six.PY2:
            with open(filename, "wb") as f:
                f.write(safe_encode(template))
        else:
            with open(filename, "w", newline="") as f:
                f.write(safe_unicode(template))
        if six.PY2:
            file = open(filename)
        else:
            file = open(filename, "rb")

        self.portal.REQUEST.form.update({
            "form.button.Import": "submit",
            "csv_upload": file,
        })
        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="subscribers-upload")
        self.assertTrue(view.__name__ == "subscribers-upload")
        view.create_subscribers()
        file.close()


class ViewsFunctionalTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



import logging
import re
from datetime import datetime

import emails
import emails.loader
import transaction
from plone import api
from plone.namedfile.scaling import ImageScale, ImageScaling
from plone.protect import PostOnly
from zope.component import getMultiAdapter, getUtility, subscribers
from zope.component.hooks import getSite

from Products.EasyNewsletter import EasyNewsletterMessageFactory as _
from Products.EasyNewsletter.behaviors.plone_user_group_recipients import (
    IPloneUserGroupRecipients,
)
from Products.EasyNewsletter.content.newsletter_issue import ISendStatus
from Products.EasyNewsletter.interfaces import IIssueDataFetcher, IReceiversPostSendingFilter
from Products.Five.browser import BrowserView
from Products.MailHost.interfaces import IMailHost

log = logging.getLogger("Products.EasyNewsletter")


image_base_url = re.compile(r"(.*@@images)\/([a-zA-Z0-9.-]*)\/?([a-zA-Z]*)")


class ENLImageScale(ImageScale):
    """override validate_access method to allow access of privat objects"""

    def validate_access(self):
        return True


class ENLImageScaling(ImageScaling):
    _scale_view_class = ENLImageScale


class PloneMessageSendMixin:
    """ """


class Message(
    PloneMessageSendMixin,
    emails.message.MessageTransformerMixin,
    emails.message.MessageSignMixin,
    emails.message.MessageBuildMixin,
    emails.message.BaseMessage,
):
    """
    Email message with:
    - DKIM signer
    - mailhost compatible send
    - Message.transformer object
    """


class LocalLoader:
    """ """

    def __getitem__(self, uri):
        image_file = None
        portal = getSite()
        pstate = getMultiAdapter((portal, portal.REQUEST), name="plone_portal_state")
        purl = pstate.portal_url()
        url_match = image_base_url.match(uri)
        if url_match:
            # @@images/71d2fe96-e930-4265-9cd8-e3d4123d75f5.jpeg
            # @@images/image
            groups = url_match.groups()
            # print(groups)
            base_url = ""
            base_url = groups[0]
            image_fieldname = groups[1]
            # @@images/image/thumb > thumb
            image_scale = groups[2]

            scaling_view = portal.unrestrictedTraverse(base_url.replace(purl, "").lstrip("/"))
            image_scale = scaling_view.publishTraverse(portal.REQUEST, image_fieldname)
            if not image_scale:
                return
            image_file = image_scale.data.open()
            image_file_data = image_file.read()
            image_file.close()
        else:
            # get the original image:
            image_obj = portal.unrestrictedTraverse(uri.replace(purl, "").lstrip("/"))
            if not image_obj:
                return
            if hasattr(image_obj, "image"):
                image_file = image_obj.image.open()
                image_file_data = image_file.read()
                image_file.close()
            elif hasattr(image_obj, "_data"):
                # OFS FSImage:
                image_file_data = image_obj._data
        # print("LocalLoader output: {0}".format(image_file_data))
        return image_file_data

    def __contains__(self, uri):
        return self.__getitem__(uri)


class NewsletterIssueSend(BrowserView):
    @property
    def is_test(self):
        return self.request.form.get("test") or False

    def __call__(self):
        """
        sets workflow state to sending and then redirects to step2 with UID as
        parameter as simple safety belt.
        """
        PostOnly(self.request)
        if self.is_test:  # test must not modify the state
            self.send()
            api.portal.show_message(
                message=_("The issue test sending has been initiated."),
                request=self.request,
            )
            return self.request.response.redirect(self.context.absolute_url())

        # XXX implement this:
        # if self.context.issue_queue is not None:
        #     self._send_issue_prepare()
        #     self.context.queue_issue_for_sendout()
        #     api.portal.show_message(
        #         message=_("The issue sending has been initiated in the background."),
        #         request=self.request,
        #     )
        #     return self.request.response.redirect(self.context.absolute_url())

        # No queuing but direct send
        # self._send_issue_prepare()
        self.send_issue_immediately()
        api.portal.show_message(
            message=_("The issue has been generated and sent to the mail server."),
            request=self.request,
        )
        return self.request.response.redirect(self.context.absolute_url())

    def _send_issue_prepare(self):
        self.request["enlwf_guard"] = True
        api.content.transition(obj=self.context, transition="send")
        # commit the transaction so that identical incoming requests, for
        # whatever reason, will not trigger another send
        transaction.commit()
        self.request["enlwf_guard"] = False

    def send_issue_immediately(self):
        """convinience view for cron and similar

        never call this from UI - needs a way to protect
        currently manager only
        """
        self._send_issue_prepare()
        self.send()

    def send(self):
        """Sends the newsletter, sending might be queued for async send out."""

        # check for workflow
        current_state = api.content.get_state(obj=self.context)
        if not self.is_test and current_state != "sending":
            raise ValueError("Executed send in wrong review state!")

        # get hold of the parent Newsletter object#
        enl = self.context.get_newsletter()
        sender_name = self.request.get("sender_name") or enl.sender_name
        sender_email = self.request.get("sender_email") or enl.sender_email
        # get Plone email_charset
        # charset = get_email_charset()
        receivers = self._get_recipients()

        # determine MailHost first (build-in vs. external)
        delivery_service_name = "mailhost"  # XXX enl.delivery_dervice
        if delivery_service_name == "mailhost":
            self.mail_host = api.portal.get_tool("MailHost")
        else:
            self.mail_host = getUtility(IMailHost, name=delivery_service_name)
        log.info('Using mail delivery service "%r"' % self.mail_host)

        send_counter = 0
        send_error_counter = 0

        issue_data_fetcher = IIssueDataFetcher(self.context)
        # get issue subject an rendered output data
        issue_data = issue_data_fetcher()
        # import pdb; pdb.set_trace()  # NOQA: E702
        for receiver in receivers:
            personalized_html = issue_data_fetcher.personalize(receiver, issue_data["body_html"])
            # get plain text version
            personalized_plaintext = issue_data_fetcher.create_plaintext_message(personalized_html)

            m = Message(
                html=personalized_html,
                text=personalized_plaintext,
                subject=issue_data["subject"],
                mail_from=(sender_name, sender_email),
                mail_to=(receiver["fullname"], receiver["email"]),
            )
            m.transformer.local_loader = LocalLoader()
            m.transform(
                images_inline=True,
                load_images=True,
                base_url=self.context.absolute_url(),
                cssutils_logging_level=logging.ERROR,
            )

            send_status = {
                "successful": None,
                "error": None,
                "datetime": datetime.now(),
            }

            msg_string = ""
            try:
                msg_string = m.as_string()
            except Exception as e:  # noqa
                send_status["successful"] = False
                send_status["error"] = e
                log.exception(
                    'Building newsletter email for "%s" failed, with error "%s"!'
                    % (receiver["email"], e)
                )
                send_error_counter += 1
                continue

            if "HTTPLoaderError" in msg_string:
                log.exception(f"Transform message failed: {m.as_string()}")

            try:
                self.mail_host.send(msg_string, immediate=True)
                send_status["successful"] = True
                log.info('Send newsletter to "%s"' % receiver["email"])
                send_counter += 1
            except Exception as e:  # noqa
                send_status["successful"] = False
                send_status["error"] = e
                log.exception(
                    'Sending newsletter to "%s" failed, with error "%s"!' % (receiver["email"], e)
                )
                send_error_counter += 1
            finally:
                receiver["status"] = send_status
        # Add information to annotations
        status_adapter = ISendStatus(self.context)
        if status_adapter:
            status_adapter.add_records(receivers)
        log.info(
            "Newsletter was sent to (%s) receivers. (%s) errors occurred!"
            % (send_counter, send_error_counter)
        )

        # change status only for a 'regular' send operation (not 'is_test')
        if not self.is_test:
            self.request["enlwf_guard"] = True
            api.content.transition(obj=self.context, transition="sending_completed")
            self.request["enlwf_guard"] = False
            self.context.setEffectiveDate(datetime.now())
            self.context.reindexObject(idxs=["effective"])
            msg_type = "info"
            additional_warning = ""
            if send_error_counter:
                msg_type = "warn"
                additional_warning = _("\nPlease check the log files, for more details!")
            api.portal.show_message(
                message=_(
                    f"Newsletter was sent to ({send_counter}) receivers. ({send_error_counter}) errors occurred!{additional_warning}"
                ),
                request=self.request,
                type=msg_type,
            )

    @property
    def salutation_mappings(self):
        """
        returns mapping of salutations. Each salutation itself is a dict
        with key as language. (prepared for multilingual newsletter)
        """
        enl = self.context.get_newsletter()
        result = {}
        lang = self.context.language or "en"

        for line in enl.salutations:
            if "|" not in line:
                continue
            key, value = line.split("|")
            result[key.strip()] = {lang: value.strip()}
        return result

    def _unique_receivers(self, receivers_raw):
        receivers = []
        mails = []
        for receiver in receivers_raw:
            if receiver["email"] in mails:
                continue
            mails.append(receiver["email"])
            receivers.append(receiver)
        return receivers

    def _get_recipients(self):
        """return list of recipients"""
        request = self.request
        enl = self.context.get_newsletter()
        salutation_mappings = self.salutation_mappings
        if self.is_test:
            # get test e-mail
            test_receiver = request.get("test_receiver", "")
            if test_receiver == "":
                test_receiver = enl.test_email
            salutation = salutation_mappings.get("default", {})
            receivers = [
                {
                    "email": test_receiver,
                    "fullname": "Test Member",
                    "salutation": salutation.get(self.context.language, ""),
                    # 'nl_language': self.language
                }
            ]
            return receivers

        # only send to all subscribers if the exclude all subscribers
        # checkbox, was not set.
        # get Subscribers
        enl_receivers = []
        if not self.context.exclude_all_subscribers:
            for subscriber_brain in api.content.find(
                portal_type="Newsletter Subscriber", context=enl
            ):
                if not subscriber_brain:
                    continue
                subscriber = subscriber_brain.getObject()
                salutation_key = subscriber.salutation or "default"
                salutation = salutation_mappings.get(salutation_key, {})
                enl_receiver = {
                    "email": subscriber.email,
                    "gender": subscriber.salutation,
                    "name_prefix": subscriber.name_prefix,
                    "firstname": subscriber.firstname or "",
                    "lastname": subscriber.lastname or "",
                    "fullname": " ".join([subscriber.firstname or "", subscriber.lastname or ""]),
                    "salutation": salutation.get(
                        None,  # subscriber.getNl_language(),
                        salutation.get(self.context.language or "en", ""),
                    ),
                    "uid": subscriber.UID(),
                    # 'nl_language': subscriber.getNl_language()
                }

                enl_receivers.append(enl_receiver)

        receivers_raw = enl_receivers

        # get subscribers over selected plone members anpid groups
        plone_receivers = []
        try:
            plone_receivers_adapter = IPloneUserGroupRecipients(self.context)
        except TypeError:
            plone_receivers_adapter = None
        if not plone_receivers_adapter:
            try:
                plone_receivers_adapter = IPloneUserGroupRecipients(enl)
            except TypeError:
                plone_receivers_adapter = None
        if plone_receivers_adapter:
            plone_receivers = plone_receivers_adapter.get_plone_subscribers()
        receivers_raw += plone_receivers
        # XXX implement this with the behavior
        # external_subscribers = self._get_external_source_subscribers(enl)
        # receivers_raw += external_subscribers
        receivers = self._unique_receivers(receivers_raw)

        # Run registered receivers post sending filters for INewsletter.
        for subscriber in subscribers([enl], IReceiversPostSendingFilter):
            receivers = subscriber.filter(receivers)

        # Run registered receivers post sending filters for INewsletterIssue.
        for subscriber in subscribers([self.context], IReceiversPostSendingFilter):
            receivers = subscriber.filter(receivers)

        return receivers



import unittest

from plone.app.testing import TEST_USER_ID, setRoles
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

from Products.CMFPlone.interfaces import INavigationSchema, ISearchSchema
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class EasyNewsletterSetupTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Folder", "test-folder")
        self.folder = self.portal["test-folder"]
        registry = getUtility(IRegistry)
        self.search_settings = registry.forInterface(ISearchSchema, prefix="plone")
        self.nav_settings = registry.forInterface(INavigationSchema, prefix="plone")

    def test_newsletter_factory(self):
        self.folder.invokeFactory("Newsletter", "newsletter")
        self.assertEqual("Newsletter", self.folder.newsletter.portal_type)

    def test_catalog(self):
        indexes = self.portal.portal_catalog.indexes()
        self.assertEqual("email" in indexes, True)

    def test_newsletter_subtypes_in_meta_types_not_to_list(self):
        self.assertTrue("Newsletter" in self.nav_settings.displayed_types)
        self.assertFalse("Newsletter Issue" in self.nav_settings.displayed_types)
        self.assertFalse("Newsletter Subscriber" in self.nav_settings.displayed_types)

    def test_newsletter_subtypes_in_types_not_searched(self):
        self.assertTrue("Newsletter Subscriber" in self.search_settings.types_not_searched)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



from plone.app.layout.viewlets.common import ViewletBase


class ENLActionsViewlet(ViewletBase):
    def update(self):
        """ """
        self.id = self.context.id
        self.enl_url = self.context.absolute_url()



from plone import api

from Products.Five.browser import BrowserView


class NewsletterIssueCopyAsDraft(BrowserView):
    def __call__(self):
        return self.copy_as_draft()

    def copy_as_draft(self):
        newsletter = self.context.get_newsletter()
        master_id = self.context.id

        if master_id.startswith("master_"):
            draft_id = master_id.replace("master_", "")
        else:
            draft_id = master_id

        draft_obj = api.content.copy(
            source=self.context, target=newsletter, safe_id=True, id=draft_id
        )

        return self.request.response.redirect(draft_obj.absolute_url() + "/edit")



import unittest

from AccessControl import Unauthorized
from App.Common import package_home
from plone import api
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, logout, setRoles
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from zope.component import getMultiAdapter, getSiteManager, getUtility

from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.tests.utils import MockMailHost
from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter.interfaces import IENLRegistrationTool
from Products.EasyNewsletter.testing import (
    PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING,
    PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING,
)
from Products.EasyNewsletter.tests.base import parsed_payloads_from_msg
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.MailHost.interfaces import IMailHost

GLOBALS = globals()
TESTS_HOME = package_home(GLOBALS)


class RegistrationIntegrationTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.portal_url()
        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.portal_workflow.setDefaultChain(
            "simple_publication_workflow",
        )
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory("Newsletter", "enl1", title="ENL 1")
        self.newsletter = self.portal.get("enl1")
        self.newsletter.senderEmail = "newsletter@acme.com"
        self.newsletter.senderName = "ACME newsletter"
        self.newsletter.testEmail = "test@acme.com"
        self.mail_settings = get_portal_mail_settings()
        # Set up a mock mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        self.portal.MailHost.smtp_host = "localhost"
        registry = getUtility(IRegistry)
        self.mail_settings = registry.forInterface(IMailSchema, prefix="plone")
        self.mail_settings.email_from_address = "portal@plone.test"
        self.mail_settings.smtp_host = "localhost"
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        # We need to fake a valid mail setup
        self.mailhost = self.portal.MailHost
        self.enl_reg_tool = getUtility(IENLRegistrationTool, "enl_registration_tool")

    def test_register_subscriber(self):
        self.assertSequenceEqual(self.mailhost.messages, [])
        self.portal.REQUEST.form.update({
            "newsletter": "/enl1",
            "salutation": "mr",
            "firstname": "Max",
            "name": "Mustermann",
            "subscriber": "max@example.com",
        })
        view = getMultiAdapter((self.portal, self.portal.REQUEST), name="register-subscriber")
        view.__call__()
        self.assertEqual(len(self.mailhost.messages), 1)
        self.assertTrue(self.mailhost.messages[0])
        msg = safe_unicode(self.mailhost.messages[0])
        parsed_payloads = parsed_payloads_from_msg(msg)
        self.assertIn("To: max@example.com", msg)
        self.assertIn("From: portal@plone.test", msg)
        self.assertIn("confirm-subscriber?hkey=", safe_unicode(parsed_payloads["text/plain"]))

        enl_reg_entry = self.enl_reg_tool.values()[0]
        self.assertEqual(
            enl_reg_entry.firstname,
            "Max",
        )
        self.assertTrue(enl_reg_entry.id)
        self.assertEqual(
            enl_reg_entry.lastname,
            "Mustermann",
        )
        self.assertEqual(
            enl_reg_entry.name_prefix,
            "",
        )
        self.assertEqual(
            enl_reg_entry.nl_language,
            "en",
        )
        self.assertEqual(
            enl_reg_entry.organization,
            "",
        )
        self.assertEqual(
            enl_reg_entry.path_to_easynewsletter,
            "enl1",
        )
        self.assertEqual(
            enl_reg_entry.salutation,
            "mr",
        )
        self.assertEqual(
            enl_reg_entry.subscriber,
            "max@example.com",
        )

    def test_confirm_subscriber(self):
        self.assertSequenceEqual(self.mailhost.messages, [])
        self.portal.REQUEST.form.update({
            "newsletter": "/enl1",
            "salutation": "mr",
            "firstname": "Max",
            "name": "Mustermann",
            "subscriber": "max@example.com",
        })
        view = getMultiAdapter((self.portal, self.portal.REQUEST), name="register-subscriber")
        view.__call__()
        enl_reg_entry = self.enl_reg_tool.values()[0]
        self.portal.REQUEST.form.update({
            "hkey": enl_reg_entry.id,
        })
        view = getMultiAdapter((self.portal, self.portal.REQUEST), name="confirm-subscriber")
        view.__call__()
        catalog = self.portal.portal_catalog
        query = {"portal_type": "Newsletter Subscriber"}
        results = catalog(query)
        self.assertTrue(len(results) == 1)
        subscriber = results[0].getObject()
        self.assertEqual(
            subscriber.firstname,
            "Max",
        )
        self.assertEqual(
            subscriber.lastname,
            "Mustermann",
        )
        self.assertEqual(
            subscriber.salutation,
            "mr",
        )
        self.assertEqual(
            subscriber.title,
            "max@example.com - Max Mustermann",
        )

        # check that anonymous can't access the subscriber object
        subscriber_uid = subscriber.UID()
        logout()
        # Unauthorized gets raised in > 5.1
        # with self.assertRaises(Unauthorized):
        try:
            subscriber_obj = api.content.get(UID=subscriber_uid)
        except Unauthorized:
            pass
        else:
            # in Plone < 5.2 subscriber_obj has to be None:
            self.assertFalse(subscriber_obj)
        login(self.portal, TEST_USER_NAME)


class RegistrationFunctionalTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.browser = Browser(self.portal)
        self.browser.handleErrors = False
        self.mail_settings = get_portal_mail_settings()
        self.mail_settings.email_from_address = "portal@plone.test"
        # Set up a mock mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        self.portal.MailHost.smtp_host = "localhost"
        self.portal.MailHost.email_from_address = "portal@plone.test"
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        # We need to fake a valid mail setup

        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # self.portal.portal_workflow.setDefaultChain(
        #     "simple_publication_workflow",
        # )
        self.portal.invokeFactory("Newsletter", "enl1", title="ENL 1")
        self.newsletter = self.portal.get("enl1")
        self.newsletter.senderEmail = "newsletter@acme.com"
        self.newsletter.senderName = "ACME newsletter"
        self.newsletter.testEmail = "test@acme.com"

        # Commit so that the test browser sees these changes
        import transaction as zt

        zt.commit()

    def test_registration_portlet(self):
        self.browser.open(self.newsletter.absolute_url())

        # self.assertTrue(
        #     u"You have been subscribed." in self.browser.contents,
        #     'There should be a portal message!')
        # self.assertTrue(
        #     subscriber1_id not in self.newsletter.objectIds(),
        #     'Subscriber should be delete now!')



from plone import api

from Products.EasyNewsletter import config
from Products.Five.browser import BrowserView


class NewsletterMasters(BrowserView):
    def __call__(self):
        return self.index()

    def get_master_issues(self):
        """return issues brains of issues in review state master"""
        enl = self.context.get_newsletter()
        issues = api.content.find(
            portal_type=config.ENL_ISSUE_TYPES,
            review_state="master",
            sort_on="modified",
            sort_order="reverse",
            context=enl,
        )
        return issues



import unittest
from datetime import datetime

from plone import api
from plone.app.testing import TEST_USER_ID, setRoles

from Products.EasyNewsletter.content.newsletter_issue import ISendStatus
from Products.EasyNewsletter.subscriber import FilterAlreadySentReceivers
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class FilterAlreadySentReceiversTests(unittest.TestCase):
    """"""

    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.newsletter = api.content.create(
            container=self.portal,
            type="Newsletter",
            id="newsletter",
        )
        self.issue = api.content.create(
            container=self.newsletter,
            type="Newsletter Issue",
            id="issue",
        )

    def test_filter(self):
        """Validate already sent emails are filtered out."""

        receivers = [
            {
                "email": "john@example.com",
                "fullname": "John Doe",
                "firstname": "John",
                "lastname": "Doe",
                "salutation": "Dear Mr.",
            },
            {
                "email": "mary@example.com",
                "fullname": "Mary Doe",
                "firstname": "Mary",
                "lastname": "Doe",
                "salutation": "Dear Mrs.",
            },
        ]

        successful_receivers = [
            {
                "email": "john@example.com",
                "fullname": "John Doe",
                "firstname": "John",
                "lastname": "Doe",
                "salutation": "Dear Mr.",
                "status": {
                    "successful": True,
                    "error": None,
                    "datetime": datetime.now(),
                },
            }
        ]

        already_sent = FilterAlreadySentReceivers(self.issue)
        final_receivers = already_sent.filter(receivers)
        self.assertEqual(len(final_receivers), 2)

        status_adapter = ISendStatus(self.issue)
        status_adapter.add_records(successful_receivers)

        final_receivers = already_sent.filter(receivers)
        self.assertEqual(len(final_receivers), 1)



from plone import api

from Products.EasyNewsletter import config
from Products.Five.browser import BrowserView


class Newsletter(BrowserView):
    # template = ViewPageTemplateFile('newsletter.pt')

    def __call__(self):
        self.issues = self.get_send_issues()
        return self.index()

    def get_send_issues(self):
        """return sended issues brains"""
        enl = self.context.get_newsletter()
        issues = api.content.find(
            portal_type=config.ENL_ISSUE_TYPES,
            review_state="sent",
            sort_on="effective",
            sort_order="reverse",
            path="/".join(enl.getPhysicalPath()),
        )
        return issues



from plone import api

# from Products.EasyNewsletter import _
from Products.EasyNewsletter import config
from Products.Five.browser import BrowserView


class NewsletterDrafts(BrowserView):
    def __call__(self):
        return self.index()

    @property
    def draft_issues(self):
        return api.content.find(
            portal_type=config.ENL_ISSUE_TYPES,
            review_state=["draft", "sending"],
            sort_on="modified",
            sort_order="reverse",
            context=self.context,
        )



from collections import namedtuple

from zope.interface import Interface

# expected types: sent=int, total=int, start=datetime
Progress = namedtuple("Progress", ["sent", "total", "start"])


class IIssueQueue(Interface):
    """Queues an issue into a queue"""

    def start(issue):
        """Queues an IENLissue for sendout"""



from plone import api
from zope.i18n import translate

from Products.EasyNewsletter import _
from Products.EasyNewsletter.utils.mail import get_portal_mail_settings
from Products.Five.browser import BrowserView

# from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class NewsletterUnsubscribeForm(BrowserView):
    def __call__(self):
        self.newsletter_url = self.context.absolute_url()
        subscriber = self.request.get("subscriber")

        if subscriber:
            self.send_unsubscribe_email(subscriber)
            return self.request.response.redirect(self.newsletter_url)
        else:
            self.form_action = self.newsletter_url + "/unsubscribe"
            return self.index()

    def send_unsubscribe_email(self, subscriber):
        newsletter = self.context
        catalog = api.portal.get_tool(name="portal_catalog")
        query = {}
        query["portal_type"] = "Newsletter Subscriber"
        query["email"] = subscriber
        results = catalog.unrestrictedSearchResults(query)
        if results:
            subscriber_brain = results[0]
            unsubscribe_url = (
                self.newsletter_url + "/unsubscribe?subscriber=" + subscriber_brain.UID
            )
            unsubscribe_header_msgid = _(
                "you_requested_to_unsubscribe",
                default="You requested to unsubscribe from the following newsletter: ${title}",
                mapping={"title": newsletter.title},
            )
            msg_text = f"{translate(unsubscribe_header_msgid)}\n{newsletter.unsubscribe_string}: {unsubscribe_url}"
            settings = get_portal_mail_settings()
            api.portal.send_email(
                recipient=subscriber,
                sender=settings.email_from_address,
                subject="{0}: {1}".format(newsletter.title, _("confirm newsletter unsubscription")),
                body=msg_text,
            )
            api.portal.show_message(
                message=_("We send you an email, please confirm this unsubscription."),
                request=self.request,
                type="info",
            )
        else:
            # todo: write an extra error msg if a plone user wants to
            # unsubscribe himself
            api.portal.show_message(
                message=_("Your email address could not be found in subscribers."),
                request=self.request,
                type="error",
            )



"""Newsletter subscriber."""

from zope.component import adapter

from Products.DCWorkflow.interfaces import IAfterTransitionEvent
from Products.EasyNewsletter.content.newsletter_issue import INewsletterIssue, ISendStatus


class FilterAlreadySentReceivers:
    """Filter all receiver which already got an email in a previous run."""

    def __init__(self, context):
        self.context = context

    def filter(self, receivers):
        status_adapter = ISendStatus(self.context)
        if not status_adapter:
            return receivers
        successful = status_adapter.get_keys(successful=True)

        return [receiver for receiver in receivers if receiver.get("email") not in successful]


@adapter(IAfterTransitionEvent)
def reset_send_status(event):
    if not INewsletterIssue.providedBy(event.object):
        return
    if event.new_state.id in ["master"]:
        status_adapter = ISendStatus(event.object)
        if not status_adapter:
            return
        status_adapter.clear()
    return



from collective.taskqueue import taskqueue
from zope.interface import implementer

from Products.EasyNewsletter.queue.interfaces import IIssueQueue

QUEUE_NAME = "Products.EasyNewsletter.queue"
VIEW_NAME = "enl_taskqueue_sendout"


@implementer(IIssueQueue)
class TCIssueQueue:
    def start(self, context):
        """Queues issue for sendout through collective.taskqueue"""
        jobid = taskqueue.add("/".join(context.getPhysicalPath() + (VIEW_NAME,)), queue=QUEUE_NAME)
        return jobid



# from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model

# from Products.EasyNewsletter import _
from zope.component import adapter
from zope.interface import Interface, implementer, provider


class IExternalDeliveryServiceMarker(Interface):
    pass


# XXX not implemented yet!
@provider(IFormFieldProvider)
class IExternalDeliveryService(model.Schema):
    """ """

    # project = schema.TextLine(
    #     title=_(u'Project'),
    #     description=_(u'Give in a project name'),
    #     required=False,
    # )


@implementer(IExternalDeliveryService)
@adapter(IExternalDeliveryServiceMarker)
class ExternalDeliveryService:
    def __init__(self, context):
        self.context = context

    # @property
    # def project(self):
    #     if hasattr(self.context, 'project'):
    #         return self.context.project
    #     return None

    # @project.setter
    # def project(self, value):
    #     self.context.project = value



# from z3c.form.browser.radio import RadioFieldWidget
from plone import api, schema
from plone.app import textfield
from plone.app.z3cform.widget import SingleCheckBoxBoolFieldWidget
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.namedfile import field as namedfile
from plone.registry.interfaces import IRegistry
from plone.supermodel import model
from z3c import relationfield
from zope.component import getUtility
from zope.interface import implementer

from Products.EasyNewsletter import _, config


def get_default_output_template():
    registry = getUtility(IRegistry)
    templates_keys = list(registry.get("Products.EasyNewsletter.output_templates"))
    if not templates_keys:
        return
    if "output_default" not in templates_keys:
        default_tmpl_key = "output_default"
    else:
        default_tmpl_key = templates_keys[0]
    return default_tmpl_key


def _get_base_path(path):
    base_obj = api.content.get(path)
    if not base_obj:
        return
    base_path = "/".join(base_obj.getPhysicalPath())
    return base_path


def get_content_aggregation_sources_base_path(context):
    return _get_base_path("/")


class INewsletter(model.Schema):
    """Marker interface and Dexterity Python Schema for Newsletter"""

    # model.fieldset(
    #     'default',
    #     label=u'Default',
    #     fields=[
    #         'sender_email',
    #         'sender_name',
    #         'test_email',
    #         'content_aggregation_sources',
    #         'output_template',
    #     ],
    # )

    model.fieldset(
        "personalization",
        label=_("Personalization"),
        fields=[
            "salutations",
            "fullname_fallback",
            "unsubscribe_string",
            "subscriber_confirmation_mail_subject",
            "subscriber_confirmation_mail_text",
            "default_prologue",
            "default_epilogue",
            "banner",
            "logo",
        ],
    )

    model.fieldset("recipients", label=_("Recipients"), fields=["exclude_all_subscribers"])

    sender_email = schema.TextLine(
        title=_("ENL_label_senderEmail", default="Sender email"),
        description=_(
            "ENL_help_senderEmail",
            default="Default for the sender address of the newsletters.",
        ),
        required=True,
    )

    sender_name = schema.TextLine(
        title=_("ENL_label_senderName", default="Sender name"),
        description=_(
            "ENL_help_senderName",
            default="Default for the sender name of the newsletters.",
        ),
        required=True,
    )

    test_email = schema.TextLine(
        title=_("ENL_label_testEmail", default="Test email"),
        description=_("ENL_help_testEmail", default="Default for the test email address."),
        required=True,
    )

    directives.widget(
        "content_aggregation_sources",
        pattern_options={
            "basePath": get_content_aggregation_sources_base_path,
            "selectableTypes": ["Collection"],
        },
    )
    content_aggregation_sources = relationfield.schema.RelationList(
        title=_(
            "ENL_content_aggregation_sources_label",
            default="Content aggregation sources",
        ),
        description=_(
            "ENL_content_aggregation_sources_desc",
            default="Choose sources to aggregate newsletter content from.",
        ),
        value_type=relationfield.schema.RelationChoice(
            title="content_aggretation_source",
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
    )

    salutations = schema.List(
        title=_("ENL_label_salutations", default="Subscriber Salutations."),
        description=_(
            "ENL_help_salutations",
            default='Define here possible salutations for subscriber. \
                One salutation per line in the form of: "mr|Dear Mr.". \
                The left hand value "mr" or "ms" is mapped to salutation \
                of each subscriber and then the right hand value, which \
                you can customize is used as salutation.',
        ),
        default=["mr|Dear Mr.", "ms|Dear Ms.", "default|Dear"],
        value_type=schema.TextLine(title="salutation"),
        required=True,
    )

    fullname_fallback = schema.TextLine(
        title=_(
            "ENL_label_fullname_fallback",
            default="Fallback for subscribers without a name.",
        ),
        description=_(
            "ENL_help_fullname_fallback",
            default="This will be used if the subscriber has no fullname.",
        ),
        default="Sir or Madam",
        required=True,
    )

    unsubscribe_string = schema.TextLine(
        title=_("ENL_label_unsubscribe_string", default="Text for the 'unsubscribe' link"),
        description=_(
            "ENL_help_unsubscribe_string",
            default="This will replace the placeholder {{UNSUBSCRIBE}}.",
        ),
        default="Click here to unsubscribe",
        required=True,
    )

    # Make sure you import: plone.namedfile
    banner = namedfile.NamedBlobImage(
        title=_("ENL_image_label", default="Banner image"),
        description=_(
            "ENL_image_desc",
            default="Banner image, you can include in the templates by"
            + "\n adding the {{banner}} placeholder into it."
            + " By default it should be 600x200 pixel.",
        ),
        required=False,
    )

    # Make sure you import: plone.namedfile
    logo = namedfile.NamedBlobImage(
        title=_("ENL_logo_label", default="Logo image"),
        description=_(
            "ENL_logo_desc",
            default="Logo image, you can include in the templates by\n"
            + " adding the {{logo}} placeholder into it.",
        ),
        required=False,
    )

    # Make sure to import: plone.app.textfield
    default_prologue = textfield.RichText(
        title=_("ENL_label_default_header", default="Prologue"),
        description=_(
            "ENL_description_text_header",
            default="The default prologue text. This is used as a default \
                for new issues. You can use placeholders like\
                {{subscriber_salutation}} and {{unsubscribe}} here.",
        ),
        default=_("{{subscriber_salutation}}<br />"),
        required=False,
    )

    # Make sure to import: plone.app.textfield
    default_epilogue = textfield.RichText(
        title=_("ENL_label_default_footer", default="Epilogue"),
        description=_(
            "ENL_description_text_footer",
            default="The default epilogue text. This is used as a default \
                for new issues. You can use placeholders like\
                {{subscriber_salutation}} and {{unsubscribe}} here.",
        ),
        default=_("<h1>Community Newsletter for Plone</h1>\n{{unsubscribe}}"),
        required=False,
    )

    # Make sure you import:
    # plone.app.z3cform.widget.SingleCheckBoxBoolFieldWidget
    directives.widget(exclude_all_subscribers=SingleCheckBoxBoolFieldWidget)
    exclude_all_subscribers = schema.Bool(
        title=_("ENL_label_excludeAllSubscribers", default="Exclude all subscribers"),
        description=_(
            "ENL_help_excludeAllSubscribers",
            default="If checked, the newsletter/mailing will not be send  \
                to all subscribers inside the newsletter. Changing this \
                setting does not affect already existing issues.",
        ),
        required=False,
        default=False,
    )

    output_template = schema.Choice(
        title=_("enl_label_output_template", default="Output template"),
        description=_(
            "enl_help_output_template",
            default="Choose the template to render the email. ",
        ),
        vocabulary="Products.EasyNewsletter.OutputTemplates",
        defaultFactory=get_default_output_template,
        required=True,
    )

    subscriber_confirmation_mail_subject = schema.TextLine(
        title=_(
            "ENL_label_subscriber_confirmation_mail_subject",
            default="Subscriber confirmation mail subject",
        ),
        description=_(
            "ENL_description_subscriber_confirmation_mail_subject",
            default="Text used for confirmation email subject. You can \
                customize the text, but it should include the \
                placeholder: ${portal_url}!",
        ),
        default=config.DEFAULT_SUBSCRIBER_CONFIRMATION_MAIL_SUBJECT,
        required=True,
    )

    subscriber_confirmation_mail_text = schema.Text(
        title=_(
            "ENL_label_subscriber_confirmation_mail_text",
            default="Subscriber confirmation mail text",
        ),
        description=_(
            "ENL_description_subscriber_confirmation_mail_text",
            default="Text used for confirmation email. You can customize \
                the text, but it should include the placeholders: \
                ${portal_url}, ${subscriber_email} and \
                ${confirmation_url}!",
        ),
        default=config.DEFAULT_SUBSCRIBER_CONFIRMATION_MAIL_TEXT,
        required=True,
    )

    directives.order_after(content_aggregation_sources="IBasic.title")
    directives.order_after(test_email="IBasic.title")
    directives.order_after(sender_name="IBasic.title")
    directives.order_after(sender_email="IBasic.title")
    directives.order_after(output_template="IRichText.text")


@implementer(INewsletter)
class Newsletter(Container):
    """ """

    def get_newsletter(self):
        return self

    # bbb to support ATCT way, needs to be removed in v5.x:
    getNewsletter = get_newsletter
